/******************************************************************************************
//
//      binfo.c
//	      Copyright (c) 2007
//		      Institute for Chemical Research, Kyoto University
//
*******************************************************************************************/
#include "linkdb.h"
#include "binfo.h"
#include "dbtab.h"
#include "version.h"

void version();
void usage();
int isdir(char *dir);
char *lowercase(char *str);
void read_dbinfotab(char *filename);
char *setcomma(char *s1, char *s2);
void report(DBNAME *dbname, char *appli);
void read_services(DBNAME *db, char *appli, char *filename);
char *getreldate(char *s1, char *s2);

int main(int argc, char *argv[]){
  char *p, *param, *opts, *db, *ext, *appli;
  char *C, *A, *T, *E, *O, *R, *I, opt;
  char dbtab_file[1024], linkdbdir[1024], basedb[32], filename[2048];
  char buffer[BUFSIZ], line[1024], services_file[1024];
  char entnum[20], resnum[20];
  char reldate[20];
  int cnt, len, i, flag;
  FILE *fp;
  DBINFO *dbi;
  static DBNAME dbname, *dbname_p, dblist, *dblist_p;
  int dbnum = 0;

  strcpy(myname, basename(argv[0]));

  p = getenv("BIOROOT");
  if( p == NULL ){
    fprintf(stderr, "BIOROOT: Undefined variable.\n");
    exit(1);
  }
  strcpy(bioroot, p);
  sprintf(databases_file, "%s%s", bioroot, databases);
  sprintf(services_file, "%s/%s", bioroot, services);

  for(cnt = 0; cnt < OPT_NUM; cnt++){
    option[cnt].param = NULL;
    option[cnt].exist = 0;
  }

  opts = binfo_opts;
  db = NULL;
  dbname_p = &dbname;
  dbnum = 0;
  appli = NULL;
  for(cnt = 1; cnt < argc; cnt++){
    if( argv[cnt][0] == '-' ){
      len = strlen(argv[cnt]);
      if( len < 2 ){
        fprintf(stderr, "Invalid option : %s\n", argv[cnt]);
        usage();
      }
      for( i = 1; i < len; i++ ){
        opt=argv[cnt][i];
        if( (param = strchr(opts, opt)) == NULL ){
          fprintf(stderr, "Invalid option : %s\n", argv[cnt]);
          usage();
        }else{
          option[(int)opt].exist = 1;
          param++;
          if( *param == ':' ){
            if( i != 1 ){
              usage();
            }
            cnt++;
            if( cnt >= argc ){
              usage();
            }
            if( argv[cnt][0] == '-' ){
              usage();
            }
            option[(int)opt].param = argv[cnt];
          }
        }
      }
    }else{
      if( ! strcmp(argv[cnt], "dbget") || ! strcmp(argv[cnt], "blast") || ! strcmp(argv[cnt], "fasta") ){
        if( appli != NULL ){
          fprintf(stderr, "Application name must be one of [dbget|fasta|blast]\n");
          usage();
        }
        appli = argv[cnt];
        continue;
      }
      if( dbnum ){
        dbname_p->next = (DBNAME *)calloc(1, sizeof(DBNAME));
        dbname_p = dbname_p->next;
      }else{
        db = argv[cnt];
      }
      dbname_p->name = argv[cnt];
      dbname_p->next = NULL;
      dbnum++;
    }
  }

  if( option['V'].exist ){
    version();
  }

  if( option['h'].exist ){
    usage();
  }

  ext = "";
  if( option['f'].exist ){
//    ext = "fdb";
    ext = "tdb";
  }

  if( option['t'].exist ){
//    ext = "tdb";
    ext = "fdb";
  }

  if( option['e'].exist ){
    ext = "edb";
  }

  read_databases(databases_file);

  if( option['f'].exist + option['t'].exist + option['e'].exist != 1 ){
    if( appli != NULL ){
      if( ! strcmp(appli, "dbget") ){
        sprintf(filename, "%s/etc/dbinfotab.ideas", bioroot);
      }else{
        sprintf(filename, "%s/etc/dbinfotab.fasta", bioroot);
      }
      read_dbinfotab(filename);
    }else{
      sprintf(filename, "%s/etc/dbinfotab.dblink", bioroot);
      read_dbinfotab(filename);
      sprintf(filename, "%s/etc/dbinfotab.ideas", bioroot);
      read_dbinfotab(filename);
      sprintf(filename, "%s/etc/dbinfotab.fasta", bioroot);
      read_dbinfotab(filename);
    }
    
    dblist_p = &dblist;
    read_services(dblist_p, appli, services_file);
    if( dbnum ){
      dbname_p = &dbname;
      report(dbname_p, appli);
    }else{
      dblist_p = &dblist;
      flag = option['n'].exist + option['r'].exist + option['d'].exist + option['l'].exist +
         option['D'].exist + option['u'].exist + option['a'].exist;
      if( flag > 0 ){
        report(dblist_p, appli);
      }else{
        setvbuf(stdout, buffer, _IOLBF, BUFSIZ);
        fprintf(stdout, "%25s *** Last database updates ***\n", "");
        fprintf(stdout, "   Date       Database             Release             #Entries       #Residues\n");
        fprintf(stdout, " --------  -------------   ------------------------  -------------  ----------------\n");
        while( dblist_p != NULL ){
          dbi = getdbinfo(databases_key(dblist_p->name), dblist_p->name);
          if( dbi->lastupdate[0] == '\0'){
            dblist_p = dblist_p->next;
            continue;
          }
          setcomma(entnum, dbi->ent_num);
          setcomma(resnum, dbi->residues_num);

          getreldate(reldate, dbi->reldate);
          fprintf(stdout, " %s  %-13.13s %17s (%6s)  %13s  %16s\n",
              dbi->lastupdate, dbi->cano, dbi->relno, reldate, entnum, resnum);
          dblist_p = dblist_p->next;
        }
      }
    }
    exit(0);
  }

  if( db == NULL ){
    sprintf(filename, "%s%s/linkdb.%s", bioroot, DATAPATH, ext);
  }else{
    dbi = getdbinfo(databases_key(db), db);
    if( dbi != NULL ){
      sprintf(linkdbdir, "%s%s/%s", bioroot, DATAPATH, dbi->cano);
    }else{
      sprintf(linkdbdir, "%s%s/%s", bioroot, DATAPATH, db);
    }
    if( isdir(linkdbdir) ){
      sprintf(dbtab_file, "%s%s", bioroot, dbtab);
      read_dbtab(dbtab_file);
      getbasedb(dbtab_key(db), db, basedb);
      if( basedb[0] == '\0' ){
        fprintf(stderr, "No such db : %s\n", db);
        exit(1);
      }
      sprintf(linkdbdir, "%s%s/%s", bioroot, DATAPATH, basedb);
      if( isdir(linkdbdir) ){
        fprintf(stderr, "No such db : %s\n", db);
        exit(1);
      }
      if( ! strcmp(basedb, "genes") || ! strcmp(basedb, "dgenes") || ! strcmp(basedb, "egenes") || ! strcmp(basedb, "mgenes") || ! strcmp(basedb, "ko")){
        sprintf(filename, "%s/%s.%s", linkdbdir, dbi->abb, ext);
      }else{
        sprintf(filename, "%s/%s.%s", linkdbdir, db, ext);
      }
    }else{
      if( dbi != NULL ){
        sprintf(filename, "%s/%s.%s", linkdbdir, dbi->cano, ext);
      }else{
        sprintf(filename, "%s/%s.%s", linkdbdir, db, ext);
      }
    }
  }

  dblist_p = &dblist;
  read_services(dblist_p, appli, services_file);
  fp = fopen(filename, "r");
  if( fp == NULL){
    exit(0);
  }
  setvbuf(fp, buffer, _IOLBF, BUFSIZ);
  while( mygetline(line, fp) != NULL){
    A=strtoken(line, "\t");
    C=strtoken(NULL, "\t");
    if( db != NULL ){
      T=strtoken(NULL, "\t");
      E=strtoken(NULL, "\t");
      O=strtoken(NULL, "\t");
      R=strtoken(NULL, "\t");
      I=strtoken(NULL, "\t");
      if( ! option['a'].exist && ! option['c'].exist && ! option['l'].exist && ! option['u'].exist){
        if( strlen(A) > 20 ){
          A[20] = '\0';
        }
        if( option['e'].exist ){
          fprintf(stdout, "%-20s %-20s %8s\n", A, C, T);
        }else{
          fprintf(stdout, "%-20s %-20s %10s %8s %8s %8s %8s\n", A, C, T, E, O, R, I);
        }
        continue;
      }
    }else{
      if( ! option['a'].exist && ! option['c'].exist && ! option['l'].exist && ! option['u'].exist ){
        if( strlen(A) > 20 ){
          A[20] = '\0';
        }
        fprintf(stdout, "%-20s %-20s\n", A, C);
        continue;
      }
    }
    if( option['a'].exist && option['c'].exist ){
      if( strlen(A) > 20 ){
        A[20] = '\0';
      }
      fprintf(stdout, "%-20s %-20s\n", A, C);
      continue;
    }

    if( option['c'].exist ){
      db = C;
    }else{
      db = A;
    }

    if( option['l'].exist ){
      dbi = getdbinfo(databases_key(C), C);
      if( dbi != NULL ){
        if( option['c'].exist ){
          fprintf(stdout, "%s\t%s\n", C, dbi->full);
        }else{
          fprintf(stdout, "%s\t%s\n", A, dbi->full);
        }
      }else{
        fprintf(stdout, "%s\n", db);
      }
      continue;

    }

    if( option['u'].exist ){
      dbi = getdbinfo(databases_key(C), C);
      if( dbi != NULL ){
        if( option['c'].exist ){
          fprintf(stdout, "%s\t%s\n", C, dbi->url);
        }else{
          fprintf(stdout, "%s\t%s\n", A, dbi->url);
        }
      }else{
        fprintf(stdout, "%s\n", db);
      }
    }else{
      fprintf(stdout, "%s\n", db);
    }
  }
  fclose(fp);

  return 0;

}

void read_dbinfotab(char *filename){
  FILE *fp;
  char buffer[BUFSIZ], line[256];
  char *db, *p;
  DBINFO *info;
  
  fp = fopen(filename, "r");
  if( fp == NULL){
    return;
  }
  setvbuf(fp, buffer, _IOLBF, BUFSIZ);
  while( mygetline(line, fp) != NULL){
    if( !isalnum((int)line[0]) )
      continue;
    db = strtoken(line, ":");
    info = getdbinfo(databases_key(db), db);
    if( info == NULL ){
      continue;
    }
    if( info->lastupdate[0] != '\0' ){
      continue;
    }
    p = strtoken(NULL, ":");
    if( p ){
      strcpy(info->relno, p);
    }
    p = strtoken(NULL, ":");
    if( p ){
      strcpy(info->reldate, p);
    }
    p = strtoken(NULL, ":");
    if( p ){
      strcpy(info->ent_num, p);
    }
    p = strtoken(NULL, ":");
    if( p ){
      strcpy(info->residues_num, p);
    }
    p = strtoken(NULL, ":");
    if( p ){
      strcpy(info->lastupdate, p);
    }
  }
  fclose(fp);
}

void read_services(DBNAME *dbname, char *appli, char *filename){
  FILE *fp, *ptr;
  char *db, *p;
  char buffer[BUFSIZ], line[256], cmd[512], incfile[512];
  DBINFO *info;
  static DBNAME *dbname_p;

  dbname_p = dbname;
  fp = fopen(filename, "r");
  setvbuf(fp, buffer, _IOLBF, BUFSIZ);
  if( fp == NULL ){
    perror(filename);
    exit(1);
  }
  while( mygetline(line, fp) != NULL){
    if( !isalnum((int)line[0]) )
      continue;
    db = strtoken2(line);
    if( ! strcmp(db, "include") ){
      p = strtoken2(NULL);
      sprintf(cmd, "echo %s", p);
      ptr = popen(cmd, "r");
      if( ptr == NULL ){
        perror(p);
        exit(1);
      }
      strcpy(incfile, mygetline(line, ptr));
      pclose (ptr);
      read_services(dbname_p, appli, incfile);
      continue;
    }
    p = strtoken2(NULL);
    if( ! p ){
      p = "noserv";
    }
    info = getdbinfo(databases_key(db), db);
    if( ! info ){
      continue;
    }
    strcpy(info->services, p);
    p = strtoken2(NULL);
    if( p ){
      strcpy(info->url, p);
    }
      
    if( appli != NULL ){
      if( strstr(info->services, appli) ){
        if( dbname_p->name != NULL ){
          dbname_p->next = (DBNAME *)calloc(1, sizeof(DBNAME));
          dbname_p=dbname_p->next;
        }
        dbname_p->name = (char *)malloc(strlen(db)+1);
        strcpy(dbname_p->name, db);
        dbname_p->next = NULL;
      }
    }else{
      if( ! strstr(info->services, "noserv") ){
        if( dbname_p->name != NULL ){
          dbname_p->next = (DBNAME *)calloc(1, sizeof(DBNAME));
          dbname_p=dbname_p->next;
        }
        dbname_p->name = (char *)malloc(strlen(db)+1);
        strcpy(dbname_p->name, db);
        dbname_p->next = NULL;
      }
    }
  }
  fclose(fp);
}

int isdir(char *dir){
  struct stat file_stat;
  char resolved_name[PATH_MAX];
  if( realpath(dir, resolved_name) != NULL ){
    if( lstat(resolved_name, &file_stat) ){
      return 1;
    }
  }
  if(! S_ISDIR(file_stat.st_mode)){
    return 1;
  }
  strcpy(dir, resolved_name);
  return 0;
}

char *lowercase(char *str){
  int idx;
  for(idx = 0; str[idx] != '\0'; idx++ ){
    if( str[idx] >= 'A' && str[idx] <= 'Z' ){
      str[idx] = str[idx] + 0x20;
    }
  }
  return str;
}

void version(){
  fprintf(stderr, "%s\n", version_string);
  exit(0);
}

void usage(){
  fprintf(stderr,"Usage : %s [-n|-r|-d|-l|-u|-D] [dbget|fasta|blast] [<db>]\n", myname);
  fprintf(stderr,"        %s [-f|-t|-e] [-l|-u] [-ac] [<db>]\n", myname);
  fprintf(stderr,"Option :\n");
  fprintf(stderr,"  -n  Display only names of databases.\n");
  fprintf(stderr,"  -r  Display only release numbers of databases.\n");
  fprintf(stderr,"  -d  Display only release dates of databases.\n");
  fprintf(stderr,"  -l  Display names and full names of databases.\n");
  fprintf(stderr,"  -u  Display names and URL of website of databases.\n");
  fprintf(stderr,"  -D  Display detailed description of databases.\n");
  fprintf(stderr,"  -a  (LinkDB) Abbreviated name of linked databases.\n");
  fprintf(stderr,"  -c  (LinkDB) Canonical name of linked databases.\n");
//  fprintf(stderr,"  -f  (LinkDB) List of source databases.\n");
//  fprintf(stderr,"  -t  (LinkDB) List of target databases.\n");
  fprintf(stderr,"  -f  (LinkDB) List of databases linked from <db>.\n");
  fprintf(stderr,"  -t  (LinkDB) List of databases linked to <db>.\n");
  fprintf(stderr,"  -e  (LinkDB) List of equivalent databases.\n");
  fprintf(stderr,"  -h  Print this message and exit.\n");
  fprintf(stderr,"  -V  Print the version number and exit.\n\n");
  exit(1);
}

char *setcomma(char *s1, char *s2){
  int len, num;
  int i, j;

  len = strlen(s2);

  if( len < 4 ){
    if( s2[0] == '0' ){
      strcpy(s1, "");
    }else{
      strcpy(s1, s2);
    }
    return s1;
  }

  num = (len-1)/3;

  j = len+num;
  s1[j] = '\0';
  j--;

  num = 0;
  for(i = len - 1; i >= 0; i--, j--){
    if(num == 3){
      s1[j] = ',';
      j--;
      num = 0;
    }
    s1[j] = s2[i];
    num++;
  }

  return s1;
}

void report(DBNAME *dbname, char *appli){
  char entnum[20], resnum[20];
  DBINFO *dbi, *subdbi;
  char *type, *p, *db;
  char reldate[20], servieces[64], fullname[128];
  int i, cnt = 0;
  FILE *fp;
  char line[128], origfile[2048];

  while( dbname != NULL ){
    dbi = getdbinfo(databases_key(dbname->name), dbname->name);
    if( dbi == NULL ){
      fprintf(stderr, "%s: %s: No such database or application\n", myname, dbname->name);
      exit(1);
    }
    if( strcmp(dbname->name, dbi->cano) ){
      dbi = getdbinfo(databases_key(dbi->cano), dbi->cano);
    }
    if( dbi->relno[0] == '\0' && dbi->lastupdate[0] == '\0'){
    dbname = dbname->next;
      continue;
    }
    if( option['n'].exist ){
      fprintf(stdout, "%s\n", dbi->cano);
      dbname = dbname->next;
      continue;
    }
    if( option['r'].exist ){
      fprintf(stdout, "%s\n", dbi->relno);
      dbname = dbname->next;
      continue;
    }
    if( option['d'].exist ){
      getreldate(reldate, dbi->reldate);
      fprintf(stdout, "%s\n", reldate);
      dbname = dbname->next;
      continue;
    }
    if( option['a'].exist ){
      if(  ! strcmp(dbi->cano, "genes") ){
        sprintf(origfile, "%s/etc/origfiles/%s.files", bioroot, dbi->cano);
        fp = fopen(origfile, "r");
        if( fp == NULL){
          return;
        }
        while( fgets(line, 128, fp) != NULL){
          line[strlen(line)-1] = '\0';
          dbi = getdbinfo(databases_key(line), line);
          strcpy(fullname, dbi->full);
          p = strstr(fullname, " KEGG");
          *p = '\0';
          fprintf(stdout, "%s\t%s\n", dbi->abb, fullname);
        }
        fclose(fp);
      }
      dbname = dbname->next;
      continue;
    }

    if( option['c'].exist ){
      db = dbi->cano;
    }else{
      db = dbi->abb;
    }
    if( option['l'].exist ){
      fprintf(stdout, "%s\t%s\n", db, dbi->full);
      dbname = dbname->next;
      continue;
    }
    if( option['u'].exist ){
      fprintf(stdout, "%s\t%s\n", db, dbi->url);
      dbname = dbname->next;
      continue;
    }
    if( cnt ){
      fprintf(stdout, "\n");
    }
    cnt++;
    setcomma(entnum, dbi->ent_num);
    setcomma(resnum, dbi->residues_num);

    fprintf(stdout, "%-16.16s %s\n", dbi->cano, dbi->full);
    fprintf(stdout, "%-16.16s Release %s,", dbi->abb, dbi->relno);
    getreldate(reldate, dbi->reldate);
    fprintf(stdout, " %s\n", reldate);

    if( dbi->orga[0] != '\0' ){
      fprintf(stdout, "%-16.16s %s\n", "", dbi->orga);
    }
    if( strcmp(resnum, "") ){
      type = NULL;
      if( dbi->type[0] == 'n'){
        type = "bases";
      }
      if( dbi->type[0] == 'p'){
        type = "residues";
      }
      if( type != NULL ){
        fprintf(stdout, "%-16.16s %s entries, %s %s\n", "", entnum, resnum, type);
      }else{
        fprintf(stdout, "%-16.16s %s entries\n", "", entnum);
      }
    }else{
      fprintf(stdout, "%-16.16s %s entries\n", "", entnum);
    }
    if( ! strcmp(dbi->cano, "aaindex") ){
      for(i = 0; i < 3; i++){
        p = (char *)aax[i];
        subdbi = getdbinfo(databases_key(p), p);
        setcomma(entnum, subdbi->ent_num);
        strcpy(fullname, subdbi->full);
        if( i == 0 ){
          p = strstr(fullname, "Index Database");
          strcpy(p, "Indices");
        }else{
          p = strstr(fullname, "Matrix Database");
          strcpy(p, "Matrices");
        }
        fprintf(stdout, "%-16.16s %7s entries for %s, %s\n",  "", entnum, subdbi->shrt, fullname); 
      }
    }
/*
    if( ! strcmp(dbi->cano, "ligand") ){
      for(i = 0; i < 6; i++){
        p = (char *)lig[i];
        subdbi = getdbinfo(databases_key(p), p);
        setcomma(entnum, subdbi->ent_num);
        strcpy(fullname, subdbi->full);
        p = strstr(fullname, "for ");
        p = p + 4;
        fprintf(stdout, "%-16.16s %7s entries for %s, %s Database\n",  "", entnum, subdbi->shrt, p); 
      }
    }
*/
    fprintf(stdout, "%-16.16s Last update: %s\n", "", dbi->lastupdate);
    if( ! appli ){
      fprintf(stdout, "%-16.16s", "");
      strcpy(servieces, dbi->services);
      p = strtoken(servieces, ",");
      while( p != NULL ){
        fprintf(stdout, " <%s>", p);
        p  = strtoken(NULL, ",");
      }
      fprintf(stdout, "\n");
    }
    dbname = dbname->next;
  }

}

char *getreldate(char *s1, char *s2){
  char reldate[20];
  char *p;
  char *Y, *M;
  int m;

  strcpy(reldate, s2);
  if( (p = strchr(reldate, '/')) != NULL ){
    Y = strtoken(reldate, "/");
    M = strtoken(NULL, "/");
    m = atoi(M);
    M = (char *)month[m];
    sprintf(s1, "%s %s", M, Y);
    return s1;
  }
  if( (p = strchr(reldate, '-')) != NULL ){
    *p = ' ';
    strcpy(s1, reldate); 
    return s1;
  }
  if( (p = strchr(reldate, '_')) != NULL ){
    *p = ' ';
    strcpy(s1, reldate); 
    return s1;
  }
  strcpy(s1, s2);
  return s2;
}
