/*******************************************************************************************
//
//      dbtab.c
//            Copyright (c) 2006 - 2007
//                    Institute for Chemical Research, Kyoto University
//
*******************************************************************************************/
#include "linkdb.h"
#include "strtoken.h"
#include "dbtab.h"

static HASH_DBTAB *hash_dbtab;

off_t dbtab_key(char *dbname){
	long	h = 0;
	char	*ptr;

	ptr = dbname;
	while (*ptr != '\0') {
		h = ((43 * h) + *ptr) % DBNUM;
		ptr++;
	}
	return h;
}

void record(HASH_DBTAB *p, DBTABINFO *dbtabinfo){
	DBLIST		*l;
	char		*c;

	p->next = (HASH_DBTAB *)calloc(1, sizeof(HASH_DBTAB));
	p->name = (char *)malloc(strlen(dbtabinfo->name)+1);
	if( p->name == NULL ){
		fprintf(stderr, "Can't allocate memory.\n");
		exit(1);
	}
	strcpy(p->name, dbtabinfo->name);
	p->type = (char *)malloc(strlen(dbtabinfo->type)+1);
	if( p->type == NULL ){
		fprintf(stderr, "Can't allocate memory.\n");
		exit(1);
	}
	strcpy(p->type, dbtabinfo->type);

	p->abb = (char *)malloc(strlen(dbtabinfo->abb)+1);
	if( p->abb == NULL ){
		fprintf(stderr, "Can't allocate memory.\n");
		exit(1);
	}
	strcpy(p->abb, dbtabinfo->abb);

	p->path = (char *)malloc(strlen(dbtabinfo->path)+1);
	if( p->path == NULL ){
		fprintf(stderr, "Can't allocate memory.\n");
		exit(1);
	}
	strcpy(p->path, dbtabinfo->path);

	p->dblist = NULL;

	if( ! strcmp(p->type, "alias") && p->path[0] != '@' ){
		p->dblist = (DBLIST *)calloc(1, sizeof(DBLIST));
		l = p->dblist;
		c = strtok(dbtabinfo->path, "+");
		if( c == NULL ){
			l->next = NULL;
			return;
		}
		l->name = (char *)malloc(strlen(c)+1);
		if( l->name == NULL ){
			fprintf(stderr, "Can't allocate memory.\n");
			exit(1);
		}
		strcpy(l->name, c);
		while(1){
			c = strtok(NULL, "+");
			if( c == NULL ){
				l->next = NULL;
				return;
			}else{
				l->next = (DBLIST *)calloc(1, sizeof(DBLIST));
				l = l->next;
				l->name = (char *)malloc(strlen(c)+1);
				strcpy(l->name, c);
			}
		}
	}

}

int insert(DBTABINFO *dbtabinfo, off_t key){
	HASH_DBTAB *p;

	p = &hash_dbtab[key];
	while(1){
		if( p->name == NULL ){
			record(p, dbtabinfo);
			return 0;
		}else{
			p = p->next;
		}
	}
	return 0;
}


void read_dbtab(char *filename){
	FILE    *fp, *ptr;
	char    line[BUFSIZ*4], readbuf[BUFSIZ];
	char    *p;
	char    cmd[1024];
	DBTABINFO	info;
	
        if( hash_dbtab == NULL ){
                hash_dbtab = (HASH_DBTAB *)calloc(DBNUM, sizeof(HASH_DBTAB));
                if( hash_dbtab == NULL ){
                        perror("hash_dbtab");
                        exit(1);
                }
	}

	info.name = (char *)malloc(1024);
	info.type = (char *)malloc(1024);
	info.path = (char *)malloc(BUFSIZ*4);
	info.abb = (char *)malloc(1024);
	fp = fopen(filename, "r");
	if( fp == NULL ){
		fprintf(stderr, "can't open file. : %s\n", filename);
		exit(1);
	}
	setvbuf(fp, readbuf, _IOLBF, BUFSIZ);
	while( mygetline(line, fp) != NULL ){
		if( ! isalnum((int)line[0]) ){
			continue;
		}

		p = strtoken2(line);
		if( p ){
			strcpy(info.name, p);
		}else{
			info.name[0] = '\0';
		}

		p = strtoken2(NULL);
		if( p ){
			strcpy(info.type, p);
		}else{
			info.type[0] = '\0';
		}

		p = strtoken2(NULL);
		if( p ){
			strcpy(info.path, p);
		}else{
			info.path[0] = '\0';
		}

		p = strtoken2(NULL);
		if( p ){
			strcpy(info.abb, p);
		}else{
			info.abb[0] = '\0';
		}

		if( ! strcmp(info.type, "include") ){
			if( strchr(info.path, '$') ){
				sprintf(cmd, "echo %s", info.path);
				if ((ptr = popen(cmd, "r")) != NULL){
					strcpy(info.path, mygetline(line, ptr));
					pclose (ptr);
				}
			}
			read_dbtab(info.path);
		}else{
			insert(&info, dbtab_key(info.name));
		}
	}
	fclose(fp);
}

char *getbasedb(off_t key, char *db, char *basedb){
	HASH_DBTAB      *p;
	DBLIST          *l;

	p = &hash_dbtab[key];
	while(1){
		if( p->name == NULL ){
			strcpy(basedb, db);
			break;
		}
		if( ! strcmp(p->name, db) ){
			if( ! strcmp(p->type, "alias") && p->path[0] != '@' ){
				l = p->dblist;
				while( l->name != NULL ){
					getbasedb(dbtab_key(l->name), l->name, basedb);
					if( basedb[0] != '\0' ){
						break;
					}
				}
				break;
			}else{
				strcpy(basedb, basename(p->path));
				break;
			}
		}else{
			p = p->next;
		}
	}
	return basedb;
}

int db_exists(char *db1, char *db2){
	char basedb[64];

	basedb[0] = '\0';
	if( ! strcmp(db1, db2) ){
		return 1;
	}

	getbasedb(dbtab_key(db2), db2, basedb);

	if( ! strcmp( db1, basedb ) ){
		return 1;
	}

	return 0;
	
}
