/******************************************************************************************
//
//      getdbname.c
//	      Copyright (c) 2006 - 2007
//		      Institute for Chemical Research, Kyoto University
//
*******************************************************************************************/
#include "linkdb.h"
#include "strtoken.h"
#include "getdbname.h"

int READ_DATABASES = 0;
static DBINFO *hash_databases;

int getdbname(DBINFO *info, char *dbname, char *filename){
	DBINFO *dbi;

	if( ! READ_DATABASES ){
		read_databases(filename);
		READ_DATABASES = 1;
	}

	dbi = getdbinfo(databases_key(dbname), dbname);
	
	if( dbi ){
		strcpy(info->cano, dbi->cano);
		strcpy(info->abb, dbi->abb);
		strcpy(info->shrt, dbi->shrt);
		strcpy(info->full, dbi->full);
		strcpy(info->orga, dbi->orga);
		strcpy(info->type, dbi->type);
		info->cano_len = dbi->cano_len;
		info->abb_len = dbi->abb_len;
	}else{
		strcpy(info->cano, dbname);
		strcpy(info->abb, dbname);
		info->cano_len = strlen(dbname);
		info->abb_len = info->cano_len;
	}
	return 0;
}

off_t databases_key(char *key){
        long    h = 0;
        char    *ptr;

        ptr = key;
        while (*ptr != '\0') {
                h = ((43 * h) + *ptr) % DBINFO_NUM;
                ptr++;
        }
        return h;
}

void record_databases(DBINFO *p, DBINFO *info){
	p->next = (DBINFO *)calloc(1, sizeof(DBINFO));
	strcpy(p->cano, info->cano);
	strcpy(p->abb, info->abb);
	strcpy(p->shrt, info->shrt);
	strcpy(p->full, info->full);
	strcpy(p->orga, info->orga);
	strcpy(p->type, info->type);
	p->cano_len = info->cano_len;
	p->abb_len = info->abb_len;

        p->relno[0] = '\0';
        p->reldate[0] = '\0';
        p->ent_num[0] = '\0';
        p->residues_num[0] = '\0';
        p->lastupdate[0] = '\0';

	p = p->next;
	p->cano[0] = '\0';
}


int insert_databases(DBINFO *dbinfo, off_t key){
        DBINFO *p;

        p = &hash_databases[key];
        while(1){
		if( p->cano[0] == '\0'){
                        record_databases(p, dbinfo);
                        return 0;
                }else{
                        p = p->next;
                }
        }
        return 0;
}

DBINFO *getdbinfo(off_t key, char *db){
	DBINFO *p;

	p = &hash_databases[key];
	while(1){
		if( p->cano[0] == '\0'){
			return NULL;
		}
		if( ! strcmp(p->cano, db) || ! strcmp(p->abb, db) ){
			return p;
		}else{
			p=p->next;
		}
	}
}

void read_databases(char *filename){
	FILE	*fp, *ptr;
	char    line[BUFSIZ], readbuf[BUFSIZ];
	char	cmd[256], incfile[512];
	char	*p;
	DBINFO	info;

	if( hash_databases == NULL ){
		hash_databases = (DBINFO *)calloc(DBINFO_NUM, sizeof(DBINFO));
		if( hash_databases == NULL ){
			perror("hash_databases");
			exit(1);
		}
	}

	fp = fopen(filename, "r");
	if( fp == NULL ){
		fprintf(stderr, "can't open file. : %s\n", filename);
		exit(1);
	}
	setvbuf(fp, readbuf, _IOLBF, BUFSIZ);
	while( fgets(line, BUFSIZ, fp) != NULL ){
		if( !isalnum((int)line[0]) )
			continue;

		strcpy(info.cano, strtoken(line, ":"));
		if( ! strncmp(info.cano, "include", 7) ){
			p = strtoken(NULL, ":");
			sprintf(cmd, "echo %s", p);
			ptr = popen(cmd, "r");
			if( ptr == NULL ){
				perror(p);
				exit(1);
			}
			strcpy(incfile, mygetline(line, ptr));
			pclose (ptr);
			read_databases(incfile);
		}else{
			p = strtoken(NULL, ":");
			strcpy(info.abb, strtoken(NULL, ":"));
			strcpy(info.shrt, strtoken(NULL, ":"));
			strcpy(info.full, strtoken(NULL, ":"));
			strcpy(info.orga, strtoken(NULL, ":"));
			strcpy(info.type, strtoken(NULL, ":"));

			info.cano_len = strlen(info.cano);
			info.abb_len = strlen(info.abb);
			insert_databases(&info, databases_key(info.cano));
			insert_databases(&info, databases_key(info.abb));
		}
	}
	fclose(fp);
}
