
#include "dbget.h"
#include "parseargs.h"
#include "readdbtab.h"
#include "netdbget.h"
#include "btit.h"

static int style_option[4] = {0};

int main(int argc, char **argv)
{
  // 環境変数 BIOROOT が必須
  string bioroot;

#ifdef REST
  bioroot = "rest";
#endif

  char *bioenv = getenv("BIOROOT");
  if( bioenv && *bioenv )
    bioroot = bioenv;

  if( bioroot.empty() ){
    cerr << "BIOROOT : Environment variable undefined." << endl;
    return 1;
  }

  // コマンドライン引数を取得する
  list<string> arguments;
  for( int i = 1 ; i < argc ; i++ ){ // argv[0] is this program
    arguments.push_back( argv[i] );
  }
  list<string> commandline = arguments;

  // 引数チェックと変数セット
  list<string> option_keys;
  option_keys.push_back("h");
  option_keys.push_back("p");
  option_keys.push_back("t");
  option_keys.push_back("d");

  multimap<string,string> options;
  list<string>::iterator aitr = arguments.begin();
  while( aitr != arguments.end() ){
    if( *aitr == "-a" ){
      options.insert(make_pair("a","a_option"));
      aitr = arguments.erase(aitr);
      continue;
    }
    if( *aitr == "-n" ){
      options.insert(make_pair("n","n_option"));
      aitr = arguments.erase(aitr);
      continue;
    }
    if( *aitr == "-s" ){
      options.insert(make_pair("s","s_option"));
      aitr = arguments.erase(aitr);
      continue;
    }
    if( *aitr == "-l" ){
      options.insert(make_pair("l","l_option"));
      aitr = arguments.erase(aitr);
      continue;
    }
    ++aitr;
  }

  int r = parseargs(arguments,option_keys,options);
  if( r == 0 || r == -1 ) usage();
  if( r == 0 ) return 1;
  if( r < 0 ) return 0;

  // 引数がひとつまたはない場合
  if( arguments.size() < 2 ){

    bool flag = true;

    if( arguments.size() == 1 ){
      aitr = arguments.begin();
      string key = *aitr;
      // 文字列にコロンを含んでいる場合
      string::size_type ret = key.find(":");
      if( ret != key.npos && ret + 1 < key.length() ){
	flag = false;
      }
    }

    if( flag ){
      string str;
      if( cin.bad() ){
	cerr << "Standard input was failed." << endl;
	return 1;
      }

      // 標準入力が端末ならメッセージを表示
      if ( isatty( STDIN_FILENO ) ){
	while(1){
	  cerr << "Entry > ";
	  getline(cin,str);
	  list<string> lin;
	  splitToList(lin,str," ");
	  lci litr = lin.begin();
	  str = *litr;
	  if( str.empty() )
	    return 0;
	  arguments.push_back(str);
	  getdata(arguments, options, commandline, bioroot);
	  arguments.pop_back();
	}
      }
      else{
	// EOFが来るまで読み込み
	while( getline(cin,str) ){
	  list<string> lin;
	  splitToList(lin,str," ");
	  lci litr = lin.begin();
	  str = *litr;
	  if( str.empty() )
	    break;
	  arguments.push_back(str);
	}
      }
    }

  }

  return getdata(arguments, options, commandline, bioroot);

}

int getdata(list<string>& arguments,
	    multimap<string,string>& options,
	    list<string>& commandline,
	    string& bioroot){

  bool verbose = false;
  bool debug   = false;
  pair<sci,sci> optp;

  bool colon = false;

  optp = options.equal_range("v");
  if( optp.first != optp.second )
    verbose = true;
  optp = options.equal_range("D");
  if( optp.first != optp.second )
    debug = true;
  optp = options.equal_range("a");
  if( optp.first != optp.second )
    style_option[0] = 1;
  optp = options.equal_range("n");
  if( optp.first != optp.second )
    style_option[1] = 1;
  optp = options.equal_range("s");
  if( optp.first != optp.second )
    style_option[2] = 1;
  optp = options.equal_range("l");
  if( optp.first != optp.second )
    style_option[3] = 1;

  if( style_option[0] == 1 && style_option[1] == 1 ){
    cerr << "btit : Cannot specify -a and -n at the same time." << endl;
    return 1;
  }
  if( style_option[0] == 1 && style_option[2] == 1 ){
    cerr << "btit : Cannot specify -a and -s at the same time." << endl;
    return 1;
  }
  if( style_option[0] == 1 && style_option[3] == 1 ){
    cerr << "btit : Cannot specify -a and -l at the same time." << endl;
    return 1;
  }
  if( style_option[1] == 1 && style_option[2] == 1 ){
    cerr << "btit : Cannot specify -n and -s at the same time." << endl;
    return 1;
  }
  if( style_option[1] == 1 && style_option[3] == 1 ){
    cerr << "btit : Cannot specify -n and -l at the same time." << endl;
    return 1;
  }
  if( style_option[2] == 1 && style_option[3] == 1 ){
    cerr << "btit : Cannot specify -s and -l at the same time." << endl;
    return 1;
  }

  string dbtype;
  if( bioroot == "rest" ) dbtype = "url";
  optp = options.equal_range("t");
  if( optp.first != optp.second ){
    sci pitr = optp.first;
    while(pitr != optp.second){
      dbtype = pitr->second;
      pitr++;
    }
  }
  string dbpath;
  if( bioroot == "rest" ) dbpath = "http://rest.genome.jp/";
  optp = options.equal_range("d");
  if( optp.first != optp.second ){
    sci pitr = optp.first;
    while(pitr != optp.second){
      dbpath = pitr->second;
      pitr++;
    }
  }
  if( dbtype.empty() && ! dbpath.empty() ){
    usage();
    return 1;
  }
  if( ! dbtype.empty() && dbpath.empty() ){
    usage();
    return 1;
  }

  list<string>::iterator aitr = arguments.begin();
  if( aitr == arguments.end() ){
    //usage();
    return 1;
  }

  if( verbose || debug )
    while( aitr != arguments.end() )
      cout << *aitr++ << endl;

  // 引数を db1:entry1 db2:entry2 ... の形式でリスト化
  string db;
  list<string> keys;
  aitr = arguments.begin();
  while( aitr != arguments.end() ){
    string key = *aitr;

    // 文字列にコロンを含んでいる場合
    string::size_type ret = key.find(":");
    if( ret != key.npos && ret + 1 < key.length() ){
      colon = true;
      keys.push_back(*aitr++);
      continue;
    }

    // コロンを使った場合、残りの文字列でもコロンを使わなければいけない
    if( colon ){
      cerr << *aitr++ << " : Syntax Error." << endl;
      cerr << "Usage : dbname:identifier" << endl;
      continue;
    }

    // 第一引数は DB名
    if( db.empty() ){
      db = *aitr++;
      continue;
    }

    // 第二引数以降はエントリ名
    key = db;
    key += ":";
    key += *aitr++;
    keys.push_back(key);
  }

  // dbtab ファイルの設定
  // デフォルト dbtab == BIOROOT/etc/dbtab
  string dbtab;
  char *dbtabenv = getenv("DBTAB");
  if( dbtabenv && *dbtabenv ){
    dbtab = dbtabenv;
  }
  optp = options.equal_range("p");
  if( optp.first != optp.second ){
    sci pitr = optp.first;
    while(pitr != optp.second){
      dbtab = pitr->second;
      pitr++;
    }
  }
  if( dbtab.length() == 0 ){
    dbtab = bioroot;
    dbtab += DBTAB;
  }

  // file name check
  if( basename(dbtab).length() == 0 ){
    cerr << "Invalid Input File " << dbtab << endl;
    return 1;
  }

  // db1:entry1 db2:entry2 ... を順番に処理
  lci kitr = keys.begin();
  if( kitr == keys.end() ){
    aitr = arguments.begin();
    cerr << *aitr << " : Syntax Error." << endl;
    cerr << "Usage : dbname:identifier" << endl;
    return 1;
  }

  list<string> inputlist;  // dbtab のデータリスト
  db = "";
  string proxy;
  bool gflag = false;  // global found flag
  while( kitr != keys.end() ){

    string key = *kitr++;
    string::size_type ret = key.find(":");
    string adb = key.substr(0,ret);
    if( adb.empty() )
      adb = key;

    // t オプションと d オプションが指定されている場合
    if( ! dbtype.empty() && ! dbpath.empty() ){
      // 初期化
      inputlist.clear();
      if( bioroot == "rest" )
	inputlist.push_back(adb + " " + dbpath + " 80");
      else
	inputlist.push_back(adb + " " + dbtype + " " + dbpath + " " + dbtype);
    }
    // dbtab の情報を取得
    else if( db.empty() || db != adb ){
      db = adb;
      if( ! readFile(dbtab,db,inputlist,proxy) ){
	cerr << db << ": No such database in dbtab." << endl;
	continue;
      }
    }

    bool flg = false; // found flag
    lci iitr = inputlist.begin();
    vector<string> lbuf;
    while( iitr != inputlist.end() ){

      // 半角スペース区切りの文字列をリストに変換
      // ex.) h.sapiens genes /bio/db/kegg/genes hsa
      splitToList(lbuf,*iitr++," ");

      vector<string>::const_iterator bitr = lbuf.begin();
      if( lbuf.size() == 4 ){
	// local database
	// ex.) /bio/db/kegg/genes/h.sapiens.tit
	string abbr = *(bitr+3);
	string index_basename = *(bitr+2);
	index_basename += "/";
	index_basename += *bitr;
	index_basename += ".tit";

	// インデックスの key は「省略名:エントリ番号」の形式
	//string ikey = *(bitr+3);
	//ikey += ":";
	//ikey += key.substr(ret+1,key.length()-ret);
	// インデックスの key はエントリ番号
	string ikey = key.substr(ret+1,key.length()-ret);

	// 既に見つかった場合は終了
	if( flg )
	  break;

	flg = display(index_basename,ikey,abbr);

      }
      else if( lbuf.size() == 3 ){
	// netdbget database
	// ex.) genbank dbget.genome.ad.jp 3266
	// ex.) prosite http://www.genome.jp/dbget-bin/dbgetserv? 80
	// netdbget(destinationhost,port,bfind,commandline,proxy);
	// 〜 Call netdbget procedure 〜
	list<string> cmd;
	cmd.push_back(key);

        if (style_option[0] == 1) cmd.push_back("-a");
        if (style_option[1] == 1) cmd.push_back("-n");
        if (style_option[2] == 1) cmd.push_back("-s");
        if (style_option[3] == 1) cmd.push_back("-l");

        string server = *(bitr+1);
	string::size_type rest = server.find("http://rest");
	int port = mystoi(*(bitr+2));
	if( rest != server.npos && rest == 0 )
	  flg = netdbget(*(bitr+1),port,"list",cmd,proxy);
	else
	  flg = netdbget(*(bitr+1),port,"btit",cmd,proxy);
	flg = true;
      }

    } // while inputlist

    if( !flg )
      cerr << key << " : No such data." << endl;
    else
      gflag = true;

  } // while keys

  if( gflag )
    return 0;
  return 1;

}

bool display(string& index_basename,string& key,string& abbr){

  string title;
  bool flg = false;

  // /bio/db/ideas/.../index_basename.tit.cdb を開く
  int fd;
  struct cdb cdb;
  struct mcdb m;
  struct mcdb_mmap map;
  bool mcdb = false;

#ifdef MCDB
  if (!open_index_mcdb(fd,&m,&map,index_basename,mcdb))
    if(!open_index(fd,&cdb,index_basename))
      return false;
#else
  if(!open_index(fd,&cdb,index_basename))
    if (!open_index_mcdb(fd,&m,&map,index_basename,mcdb))
      return false;
#endif

  // index_basename.tit.cdb は
  // key に対応するデータがタイトル情報になるように作成されている
  if (mcdb) {
    flg = get_value_mcdb(&m,&map,key,title);
    close_index_mcdb(fd,&map);
  }
  else {
    flg = get_value(&cdb,key,title);
    close_index(fd,&cdb);
  }

  if (!flg)
    return false;

  // タイトルデータを表示
  if (style_option[2] == 1 || style_option[3] == 1) // -s or -l option
    return display_alt(title,abbr);

  if (style_option[0] == 1) { // -a option

    // 2つめの単語が [xxxxx] という形式だったら
    // xxxxx が Accession と仮定
    // なければデフォルト形式で表示
    list<string> wordlist;
    splitToList(wordlist,title," ");
    lci witr = wordlist.begin();

    if (++witr != wordlist.end()) {
      string acc = *witr;
      string::size_type ret1 = acc.find("[");
      string::size_type ret2 = acc.find("]");

      if (ret1 != acc.npos && ret1 == 0 &&
	  ret2 != acc.npos && ret2 == (acc.length()-1)) {
	acc = acc.substr(ret1+1,ret2-1);

	// この acc が Accession かどうかを確かめる
	// acc がインデックスのキーになっていれば Accession
	// そうでなければデフォルト形式で表示
	flg = false;
	string ac = acc;
	string dummy;

#ifdef MCDB
        if (open_index_mcdb(fd,&m,&map,index_basename,mcdb)) {
          flg = get_value_mcdb(&m,&map,ac,dummy);
          close_index_mcdb(fd,&map);
        }
        else if (open_index(fd,&cdb,index_basename)) {
	  flg = get_value(&cdb,ac,dummy);
          close_index(fd,&cdb);
        }
#else
        if (open_index(fd,&cdb,index_basename)) {
	  flg = get_value(&cdb,ac,dummy);
          close_index(fd,&cdb);
        }
        else if (open_index_mcdb(fd,&m,&map,index_basename,mcdb)) {
          flg = get_value_mcdb(&m,&map,ac,dummy);
          close_index_mcdb(fd,&map);
        }
#endif

	if (flg) {
	  // 最初の単語
	  string id = title;
	  string content;
	  string::size_type ret = title.find(" ");
	  if( ret != title.npos )
	    id = title.substr(0,ret);

	  // 最初の単語はコロンを含んでいる
	  ret = id.find(":");
	  if( ret != id.npos && ret + 1 < id.length() ){
	    content = title.substr(ret+1);
	    title = id.substr(0,ret);
	  }
	  // コロンを含んでいなければ省略名を使う
	  else{
	    content = title;
	    title = abbr;
	  }
	  title += ":";
	  title += acc;
	  title += " ";
	  title += content;
	}
      }
    }
  }

  if( style_option[1] == 1 ){ // -n option
    // 最初の単語
    string id = title;
    string::size_type ret = title.find(" ");
    if( ret != title.npos )
      id = title.substr(0,ret);
    // 最初の単語がコロンを含んでいる場合
    ret = id.find(":");
    if( ret != id.npos && ret + 1 < id.length() )
      title = title.substr(ret+1);
    // コロンを含んでいなければ何もしない
  }

  cout << title << endl;

  return true;

}

bool display_alt(string& title,string& abbr){

  list<string> wordlist;
  splitToList(wordlist,title," ");
  lci witr = wordlist.begin();

  // 最初の単語はコロンを含んでいる
  string id = *witr++;
  string::size_type ret = id.find(":");
  if( ret != id.npos && ret + 1 < id.length() )
    id = id.substr(ret+1);

  // 前置きとして db:id を出力
  cout << abbr << ":" << id;

  // -s option では前置きとして db:id id を出力
  if( style_option[2] == 1 ) // -s option
    cout << " " << id;

  // path、ec、md、ds、rn、rp、rc は id のみ出力する
  if( abbr == "path" || abbr == "rn" || abbr == "ec" ||
      abbr == "rp" || abbr == "rc" || abbr == "md" ||
      abbr == "ds" || abbr == "ds_ja" ){
    cout << endl;
    return true;
  }

  // GENES 判定
  bool is_genes = false;
  if (witr != wordlist.end()) {
    string word = *witr++;

    // no KO assigned ではじまる場合は GENES 判定
    if (word == "no") {
      word = *witr++;
      if (witr != wordlist.end()) {
        if (word == "KO") {
          word = *witr++;
          if (witr != wordlist.end()) {
            if (word == "assigned") {
              is_genes = true;
            }
          }
        }
      }
    }

    // K番号ではじまり、単独の | がある場合も GENES 判定
    else {
      bool knumber = false;
      int len = word.length();
      int i = 1;
      if (word[0] == 'K' && len == 6)
        for (i = 1; i < 6; ++i)
          if(! isdigit(word[i]))
            break;
      if (i == 6)
        knumber = true;

      if (knumber) {
        while (witr != wordlist.end()){
          word = *witr++;
          if (word == "|") {
            is_genes = true;
            break;
          }
        }
      }
    }

    witr = wordlist.begin();
    witr++;
  }

  // ふたつめの単語以降を取得
  // K番号があればそこから後ろは削除
  bool pipe = false;
  list<string> titlelist;
  while (witr != wordlist.end()) {
    string word = *witr++;

    // GENES は必ず | があり、その後ろに丸括弧で囲まれた DB名がある
    // さらにその後ろに NAME と DEFINITION がある
    if (is_genes && !pipe) {
      if (word == "|") {
        pipe = true;

        // | の次の単語が丸括弧で閉じられているなら DB名
        // (DB名) をスキップ
        if (witr != wordlist.end()) {
          string wrd = *witr;
          if (wrd[wrd.length()-1] == ')')
          witr++;
        }

      }
      continue;
    }

    int len = word.length();
    int i = 1;
    if( word[0] == 'K' && len >= 6 )
      for( i = 1; i < 6; ++i )
	if( ! isdigit(word[i]) )
	  break;
    if( len > 6 && isdigit(word[i+1]) ) i = 1;
    if( i == 6 ){
      // KO の -l option
      // KO は必ず NAME行を持ち K番号が記載されている
      // KO で -l が指定されたとき NAME を出さず、DEFINITION を出す
      if( abbr == "ko" && style_option[3] == 1 ){
	continue;
      }
      else{
	break;
      }
    }
    titlelist.push_back(word);
  }

  // セミコロン+スペースがある場合はセミコロンまでを name に取り出す
  // そうでない場合は全部を name と def に取り出す
  unsigned counter = 0;
  lci titr = titlelist.begin();
  bool semicolon = false;
  string name;
  string def;
  while( titr != titlelist.end() ){
    ++counter;
    string word = *titr++;
    unsigned len = word.length();
    ret = word.rfind(";");
    if( ret != word.npos && ret == len-1 ){
      word = word.substr(0,ret);

      // 文末にセミコロンがある場合 semicolon は false
      if( counter != titlelist.size() )
	semicolon = true;
    }
    name += word;
    name += " ";

    // セミコロンがあればその後ろを def に取り出す
    if( semicolon ){
      def.erase();
      while( titr != titlelist.end() ){
	def += *titr++;
	def += " ";
      }
      break;
    }

    // KO は必ず NAME行を持つ
    // セミコロンのないエントリは NAME のみ定義されていて DEFINITION が未定義
    // GENES/GENOME は必ず DEFINITION行を持つ
    // セミコロンのないエントリは DEFINITION のみ定義されていて NAME が未定義
    // 他のデータベースでは def を出力しないので def の値は実行結果に影響しない
    if( abbr != "ko" ){
      def += word;
      def += " ";
    }

  }

  // name から行末のスペースとセミコロンを削除
  int len = name.length();
  while( len > 0 && (name[len-1] == ' ' || name[len-1] == ';') )
    name = name.substr(0,--len);

  // def から行末のスペースとセミコロンを削除
  len = def.length();
  while( len > 0 && (def[len-1] == ' ' || def[len-1] == ';') )
    def = def.substr(0,--len);

  if( ! name.empty() ){

    // cpd、dr、ev、gl の場合は name 全部を小括弧で囲んで表示
    // これらは必ず NAME行を持つ
    // NAME はセミコロンで区切られているので name には最初の synonym が取り出されている
    if( abbr == "cpd" || abbr == "dr"  || abbr == "ev" || abbr == "gl" ||
	abbr == "cpd_ja" || abbr == "dr_ja" || abbr == "ev_ja" ){

      if( style_option[2] == 1 ) // -s option
	cout << " (" << name << ")" << endl;
      else
	cout << " " << name << endl;

      return true;
    }

    // カンマをデリミタとしてリスト化
    splitToList(wordlist,name,",");
    witr = wordlist.begin();

    // KO は必ず NAME行を持つ
    // セミコロンのないエントリは NAME のみ定義されていて DEFINITION が未定義
    // NAME はカンマで区切られているので *witr には最初の synonym が取り出されている
    if( abbr == "ko" ){

      if( style_option[2] == 1 ) // -s option
	cout << " (" << *witr << ")";
      else
	cout << " " << *witr;

      // -l option のときは DEFINITION を出す
      if( style_option[3] == 1 && len > 0 )
	cout << "; " << def;

      cout << endl;
      return true;
    }

    // GENES/GENOME は必ず DEFINITION行を持つ
    // 基本的に semicolon が true なら NAME データを持つ
    // NAME はカンマで区切られているので *witr には最初の synonym が取り出されている
    // 1) NAME がなく、DEFINITION の文中にセミコロンがある場合も semicolon は true になる
    //    *witr にスペースが含まれているか、もしくは単語ひとつの場合は synonym ではない
    // 2) NAME が定義されていて DEFINITION が空のものがあると正しい結果が得られない
    //    → GENES に 1エントリだけある
    bool synonym = false;
    if( semicolon ){
      string word = *witr;
      if( word.find(" ") == word.npos && titlelist.size() > 1 )
	synonym = true;
    }

    if( synonym ){
      if( style_option[2] == 1 ){ // -s option
	cout << " (" << *witr << ")";
      }
      else{
	cout << " " << *witr;
      }
    }

    // -l option のときは DEFINITION を出す
    if( style_option[3] == 1 && len > 0 ){
      // synonym があればセミコロンでつなぐ
      if( synonym )
	cout << ";";
      cout << " " << def;
    }

  }

  cout << endl;
  return true;
}
