
#include "dbget.h"

void splitToList(vector<string>& listc,
		 const string& str,
		 const string& delim)
{
  // 初期化
  listc.clear();
  unsigned pos = 0;
  unsigned len = str.length();
  string::size_type ret = str.find(delim);
  while( ret != str.npos && ret + 1 < len ){
    if( pos != ret )
      listc.push_back(str.substr(pos,ret-pos));
    pos = ret+1;
    ret = str.find(delim,pos);
  }
  if( pos != ret ){
    if( ret != str.npos )
      listc.push_back(str.substr(pos,ret-pos));
    else
      listc.push_back(str.substr(pos,str.length()-pos));
  }
}

void splitToList(list<string>& listc,
		 const string& str,
		 const string& delim)
{
  // 初期化
  listc.clear();
  unsigned pos = 0;
  unsigned len = str.length();
  string::size_type ret = str.find(delim);
  while( ret != str.npos && ret + 1 < len ){
    if( pos != ret )
      listc.push_back(str.substr(pos,ret-pos));
    pos = ret+1;
    ret = str.find(delim,pos);
  }
  if( pos != ret ){
    if( ret != str.npos )
      listc.push_back(str.substr(pos,ret-pos));
    else
      listc.push_back(str.substr(pos,str.length()-pos));
  }
}

void trimSpaces(string& str){
  list<string> lstr;
  splitToList(lstr,str," ");
  list<string>::const_iterator itr = lstr.begin();
  str.erase();
  for( ; itr != lstr.end(); ++itr )
    str += *itr;
}

void trimChars(string& str,const string& delim){
  list<string> lstr;
  splitToList(lstr,str,delim);
  list<string>::const_iterator itr = lstr.begin();
  str.erase();
  for( ; itr != lstr.end(); ++itr )
    str += *itr;
}

string trimLeftSpaces(const string& str){
  int len = str.length();
  int i = 0;
  for( ; i < len; ++i )
    if( ! isspace(str[i]) )
      break;
  return str.substr(i);
}

string trimRightSpaces(const string& str){
  int len = str.length();
  int i = --len;
  for( ; i > 0 ; --i )
    if( ! isspace(str[i]) )
      break;
  return str.substr(0,++i);
}

string dirname(const string& file){
  string::size_type ret = file.rfind("/");
  return ( ret == file.npos ) ? "" : file.substr(0,ret);
}

string basename(const string& file){
  list<string> dirname;
  splitToList(dirname,file,"/");
  list<string>::const_reverse_iterator bitr = dirname.rbegin();
  return ( (*bitr)[(*bitr).length()-1] == '/' ) ? "" : *bitr;
}

void Lower(string& str){
  int length = str.length();
  for( int i = 0; i < length; i++ ){
    if( str[i] >= 0x41 && str[i] <= 0x5a ){
      str[i] += 0x20;
    }
  }
}

void Upper(string& str){
  int length = str.length();
  for( int i = 0; i < length; i++ ){
    if( str[i] >= 0x61 && str[i] <= 0x7a ){
      str[i] -= 0x20;
    }
  }
}

void cat(const string& filename){

#ifdef SOL
  int fd = open64(filename.c_str(),O_RDONLY);
#else
  int fd = open(filename.c_str(),O_RDONLY);
#endif

  if( fd > -1 ){

    char buf[MAXSIZE+1] = {0};
    while(1){
      int n = read(fd,buf,MAXSIZE);
      if( n == 0 )
	break;
      if( write(1,buf,n) < 0 )
	break;
    }
    close(fd);

  }

}

string itos(const int& num){
  // string stream for conversion from int to string
#ifdef CC_VERSION2
  strstream sstr;
#else
  stringstream sstr;
#endif
  sstr << num;
  string snum = sstr.str(); // int to string;
  return snum;
}

bool isTNumber(const string& str){
  int rtn = 0;
  if( str.length() == 6 )
    if( str[0] == 'T' || str[0] == 't' )
      for(int i=1;i<6;++i)
	if(isdigit(str[i]))
	  ++rtn;
  if(rtn==5) return true;
  return false;
}

bool isRGNumber(const string& str){
  int rtn = 0;
  if( str.length() == 5 )
    if( str[0] == 'R' || str[0] == 'r' )
      if( str[1] == 'G' || str[1] == 'g' )
        for(int i=2;i<5;++i)
          if(isdigit(str[i]))
            ++rtn;
  if(rtn==3) return true;
  rtn=0;
  if( str.length() == 8 )
    if( str[0] == 'R' || str[0] == 'r' )
      if( str[1] == 'G' || str[1] == 'g' )
        for(int i=2;i<5;++i)
          if(isdigit(str[i]))
            ++rtn;
  if(rtn==3)
    if( str[5] == '.' || str[5] == '_' )
      for(int i=6;i<8;++i)
        if(isdigit(str[i]))
          ++rtn;
  if(rtn==5) return true;
  return false;
}

bool isECNumber(const string& str){

  // EC number is in the form of 1.2.3.4
  // The first digit has to be in the range of 1-7.
  int i = 0, rtn = 0, len = 0, dots = 0;
  int dot_flg = 0, dash_flg = 0;

  len = str.length();
  if(len < 7) return false;
  if(!isdigit(str[0])) return false;
  if(str[0] > '7' || str[0] < '1' ) return false; // inhibit out of range.
  if(str[1] != '.') return false; // has to be a dot.
  if(str[len-1] == '.') return false; // does not end with a dot.

  for(i=0;i<len;++i){
    if( isdigit(str[i]) ){
      if( dash_flg ) return false; // inhibit numeric number after a dash.
      if( dot_flg && str[i] == '0' ) return false; // inhibit 0 after a dot.
      ++rtn;
      dot_flg = 0;
      dash_flg = 0;
    }
    else if( str[i] == '-' ){
      if( dash_flg++ ) return false; // inhibit consecutive dashes.
      ++rtn;
      dot_flg = 0;
    }
    else if(str[i] == '.' ){
      if( dot_flg++ ) return false; // inhibit consecutive dots.
      ++rtn;
      ++dots;
      dash_flg = 0;
    }
  }

  if( dots != 3 ) return false;
  if( rtn != len ) return false;

  return true;
}

// CDB ファイルを開いて初期化
bool open_index(int& fd, struct cdb *cdbp, const string& file){

  // インデックスファイルのフルパス
  string cdbfile = file;
  cdbfile += ".cdb";

  fd = open(cdbfile.c_str(), O_RDONLY);
  if (fd < 0)
    return false;

  // インデックスファイルのサイズチェック
  struct stat st;
  fstat(fd,&st);
  if (st.st_size >= UINT_MAX)
    return false;

  if (!init_index(fd,cdbp))
    return false;

  return true;
}

// CDB 構造体を初期化
bool init_index(int& fd, struct cdb *cdbp){

  if (cdb_init(cdbp, fd) < 0) {
    cerr << "cdb initialize error." << endl;
    return false;
  }

  return true;
}

// key を探索し、対応する値を取得する
bool get_value(struct cdb *cdbp,string& key,string& value){

  // インデックスのキーは db:entry 形式の ID で
  // ASCII小文字で記載されている
  Lower(key);

  // dbtab のインデックスのキーはファイル名と同じ
  // このため T番号は大文字
  if( value.length() > 0 )
    if( value == "dbtab" )
      if( isTNumber(key) || isRGNumber(key) )
	Upper(key);

  // value;
  // ex.) dr:d00100 /bio/db/ideas/ligand/drug 156698 2554
  char *data;
  unsigned keylen, datalen;
  keylen = key.length();

  // データがなければ false を返す
  if (cdb_find(cdbp, key.c_str(), keylen) > 0) {
    datalen = cdb_datalen(cdbp);
    data = (char *)malloc(datalen + 1);
    cdb_read(cdbp, data, datalen, cdb_datapos(cdbp));
    data[datalen] = '\0';
    value = data;
    free(data);
    return true;
  }

  return false;

  /***** simplistic query mode
  fd = open(filename, O_RDONLY);
  if (cdb_seek(fd, key, keylen, &datalen) > 0) {
    data = malloc(datalen + 1);
    cdb_bread(fd, data, datalen);
    data[datalen] = '\0';
    printf("key=%s data=%s\n", key, data);
  }
  else
    printf("key=%s not found\n", key);
  close(fd);
  ***************/
}

// CDB ファイルを閉じる
void close_index(int fd, struct cdb *cdbp){
  cdb_free(cdbp);
  close(fd);
}

// MCDB ファイルを開いて初期化
bool open_index_mcdb(int& fd, struct mcdb *mp,
                     struct mcdb_mmap *mapp,
                     const string& file,
                     bool& mcdb){
  // インデックスファイルのフルパス
  string cdbfile = file;
  cdbfile += ".mcdb";

  // open mcdb
  fd = nointr_open(cdbfile.c_str(), O_RDONLY, 0);
  if (fd == -1)
    return false;
  mcdb = true;

  if (!init_index_mcdb(fd,mp,mapp))
    return false;

  return true;
}

// MCDB 構造体を初期化
bool init_index_mcdb(int& fd, struct mcdb *mp,
                     struct mcdb_mmap *mapp){

  memset(mapp, '\0', sizeof(*mapp));
  int rv = mcdb_mmap_init(mapp, fd);
  (void) nointr_close(fd);
  if (!rv) return false;
  memset(mp, '\0', sizeof(*mp));
  mp->map = mapp;

  return true;
}

// key を探索し、対応する値を取得する
bool get_value_mcdb(struct mcdb *mp, struct mcdb_mmap *mapp,
                    string& key,string& value){
  // インデックスのキーは db:entry 形式の ID で
  // ASCII小文字で記載されている
  Lower(key);

  // dbtab のインデックスのキーはファイル名と同じ
  // このため T番号は大文字
  if (value.length() > 0)
    if (value == "dbtab")
      if (isTNumber(key)||isRGNumber(key))
	Upper(key);

  // value;
  // ex.) dr:d00100 /bio/db/ideas/ligand/drug 156698 2554
  unsigned keylen = key.length();
  char *data;
  if (mcdb_findstart(mp, key.c_str(), keylen)) {
    bool rc = mcdb_findnext(mp, key.c_str(), keylen);
    if (rc) {
      data = (char *)mcdb_dataptr(mp);
      value = data;
      return true;
    }
  }
  return false;
}

// MCDB ファイルを閉じる
void close_index_mcdb(int fd, struct mcdb_mmap *mapp){
  mcdb_mmap_free(mapp);
  close(fd);
}
