
#include "dbget.h"
#include "epd.h"

using namespace std;

Epd::Epd():
    _entry("ID"),
    _ac("AC"),
    _de("DE"),
    _os("OS"),
    _do("DO        Expression/Regulation:"),
    _delim("//"),
    _title_ac(""),
    _title_de(""),
    _title_os(""),
    _title_do(""),
    _ntseq("SQ"),
    _ntseq_id(""),
    _ntseq_offset(0),
    _ntseq_size(0),
    _ntseqlen(0){}

Epd::~Epd(){}

void Epd::parser(ifstream& ifs,
		       string& buf){

  if( buf.find(_entry) == 0 ){
    set_offset(ifs,buf);
    _header = _entry;
    buf = buf.substr(_header.length());
    set_id(buf);
  }

  else if( buf.find(_ac) == 0 ){
    _header = _ac;
    buf = buf.substr(_header.length());

    // accession からセミコロンを除く
    // ex.) AC   P15455; Q3E711; Q9FFH7;
    //      → P15455 Q3E711 Q9FFH7
    buf = trimLeftSpaces(buf);
    trimChars(buf,";");

    // スペースで連結
    if( ! _accession.empty() )
      _accession += " ";
    _accession += buf;

    // タイトルに使用する accession はプライマリのみ
    if( _title_ac.empty() ){
      // 半角スペースで区切られた最初の単語
      string::size_type ret = buf.find(" ");
      if( ret != buf.npos )
	buf = buf.substr(0,ret);
      _title_ac = buf;
    }

  }

  else if( buf.find(_de) == 0 ){
    _header = _de;
    buf = buf.substr(_header.length());

    if( ! _title_de.empty() )
      extend_title(_title_de,buf);
    else
      _title_de = trimLeftSpaces(buf);
  }

  else if( buf.find(_os) == 0 ){
    _header = _os;
    buf = buf.substr(_header.length());

    if( ! _title_os.empty() )
      extend_title(_title_os,buf);
    else
      _title_os = trimLeftSpaces(buf);
  }

  else if( buf.find(_do) == 0 ){
    _header = _do;
    buf = buf.substr(_header.length());

    if( ! _title_do.empty() )
      extend_title(_title_do,buf);
    else
      _title_do = trimLeftSpaces(buf);
  }

  else if( buf.find(_ntseq) == 0 ){
    _header = _ntseq;
    _ntseq_offset = ifs.tellg();
  }

  // 何かの継続行
  // 先頭が半角スペースの行は SQ の継続行
  else if( buf.find(" ") == 0 ){
    // 配列データの処理
    // ※ この統計処理は seqnew の実行結果に出るのみ
    if( _header == _ntseq ){
      trimSpaces(buf);
      int len = buf.length();

      // このエントリの塩基配列の長さ
      _ntseqlen += len;

      // データベース全体の塩基配列の長さ
      _total_ntseqlen += len;

      // 最長の塩基配列
      if( _max_ntseqlen < _ntseqlen )
	_max_ntseqlen = _ntseqlen;

      // 最長の配列(GENESならアミノ酸と塩基の配列の長さの合計)
      _max_seqlen = _max_ntseqlen;

    }
  }

  // エントリ終了処理
  else if( buf.find(_delim) == 0 ){
    set_size(ifs);

    // ここまでが SQ行なら、ここで配列データの統計情報をセット
    if( _header == _ntseq )
      set_seq_size(buf,_ntseq_size,_ntseq_offset);

    writer();
  }

  // 何かのヘッダ行
  else{

    // ここまでが SQ行なら、ここで配列データの統計情報をセット
    if( _header == _ntseq ){
      _size = ifs.tellg();
      _size -= _offset;
      set_seq_size(buf,_ntseq_size,_ntseq_offset);
    }

    set_header(buf);
  }

}

void Epd::writer(){

  // プライマリ accession は大括弧で括る
  if( ! _title_ac.empty() ){
    _title = "[";
    _title += _title_ac;
    _title += "]";
  }

  // accession の後ろは半角スペースで連結
  if( ! _title.empty() && ! _title_de.empty() )
    _title += " ";

  _title += _title_de;

  if( ! _title.empty() && ! _title_os.empty() )
    _title += "; ";

  _title += _title_os;

  if( ! _title.empty() && ! _title_do.empty() )
    _title += "; ";

  _title += _title_do;

  _ntseq_id = "_ntseq_";
  _ntseq_id += _id;

  Seqnew::writer();

  if( _ntseq_offset > 0 )
    Seqnew::writer(_ntseq_id,_ntseq_offset,_ntseq_size);

  clear();
}

void Epd::clear(){

  Seqnew::clear();

  _title_ac = "";
  _title_de = "";
  _title_os = "";
  _title_do = "";
  _ntseq_id = "";
  _ntseq_offset = 0;
  _ntseq_size = 0;
  _ntseqlen = 0;

}
