
#include "dbget.h"
#include "genes.h"

using namespace std;

Genes::Genes():
    _entry("ENTRY"),
    _symbol("SYMBOL"),
    _code("ORG_CODE"),
    _name("NAME"),
    _disease("DISEASE"),
    _orthology("ORTHOLOGY"),
    _delim("///"),
    _title_symbol(""),
    _title_name(""),
    _title_original_db(""),
    _title_disease(""),
    _title_orthology(""),
    _aaseq("AASEQ"),
    _aaseq_id(""),
    _aaseq_offset(0),
    _aaseq_size(0),
    _aaseqlen(0),
    _ntseq("NTSEQ"),
    _ntseq_id(""),
    _ntseq_offset(0),
    _ntseq_size(0),
    _ntseqlen(0){}

Genes::~Genes(){}

void Genes::set_is_genes(string& flatfile, string& abbr){
  _is_genes = false;
  if (flatfile.find("T0") != flatfile.npos ||
      abbr == "ag" || abbr == "vg" || abbr == "vp")
    _is_genes = true;
}

void Genes::parser(ifstream& ifs,
		   string& buf){

  if( buf.find(_entry) == 0 ){
    set_offset(ifs,buf);
    _header = _entry;
    buf = buf.substr(_header.length());
    set_id(buf);
  }

  else if( buf.find(_symbol) == 0 ){
    // T49000 の SYMBOL行は名前が重複しているので除外する
    if (_abbr != "vp") {
      _header = _symbol;
      buf = buf.substr(_header.length());
      _title_symbol = trimLeftSpaces(buf);

      // accession からカンマを除く
      _accession = _title_symbol;
      string::size_type ret = _accession.find(",");
      while( ret != _accession.npos ){
        _accession.erase(ret,1);
        ret = _accession.find(",");
      }
    }
  }

  else if( buf.find(_code) == 0 ){ // for GENOME
    // SYMBOL行と同じ処理
    _header = _code;
    buf = buf.substr(_header.length());
    _title_symbol = trimLeftSpaces(buf);

    // accession からカンマを除く
    _accession = _title_symbol;
    string::size_type ret = _accession.find(",");
    while( ret != _accession.npos ){
      _accession.erase(ret,1);
      ret = _accession.find(",");
    }
  }

  else if( buf.find(_name) == 0 ){
    _header = _name;
    buf = buf.substr(_header.length());
    _title_name = trimLeftSpaces(buf);

    // 最初の括弧を除く
    string::size_type ret1 = _title_name.find("(");
    string::size_type ret2 = _title_name.find(")");
    if (ret1 != _title_name.npos && ret2 != _title_name.npos) {
      if (ret1 == 0) {
        if (_is_genes) {
          // Original DB 取り出し
          _title_original_db = _title_name.substr(ret1,ret2-ret1+1);
          buf = _title_name;
          buf.erase(ret1,ret2-ret1+1);
          _title_name = trimLeftSpaces(buf);
        }
      }
    }

  }

  /*
  else if( buf.find(_disease) == 0 ){
    _header = _disease;
    buf = buf.substr(_header.length());
    _title_disease = trimLeftSpaces(buf);
  }
  */

  else if( buf.find(_orthology) == 0 ){
    _header = _orthology;
    buf = buf.substr(_header.length());
    _title_orthology = trimLeftSpaces(buf);

    // ex.)
    // KO: K04180  chemokine (C-C motif) receptor 5
    // -> K04180 chemokine (C-C motif) receptor 5
    string::size_type ret = _title_orthology.find("KO: ");
    if( ret != _title_orthology.npos )
      _title_orthology.erase(ret,4);
    ret = _title_orthology.find("  ");
    while( ret != _title_orthology.npos ){
      _title_orthology.erase(ret,1);
      ret = _title_orthology.find("  ");
    }
  }

  else if( buf.find(_aaseq) == 0 ){
    _header = _aaseq;
    _aaseq_offset = ifs.tellg();
  }

  else if( buf.find(_ntseq) == 0 ){
    _header = _ntseq;
    _ntseq_offset = ifs.tellg();

    _size = ifs.tellg();
    _size -= _offset;
    set_seq_size(buf,_aaseq_size,_aaseq_offset);
  }

  // 何かの継続行
  else if( buf.find("   ") == 0 ){
    if( _header == _symbol ){
      extend_title(_title_symbol,buf);

      // accession からカンマを除く
      string _accession_symbol = trimLeftSpaces(buf);
      string::size_type ret = _accession_symbol.find(",");
      while( ret != _accession_symbol.npos ){
	_accession_symbol.erase(ret,1);
	ret = _accession_symbol.find(",");
      }
      // スペースで連結
      _accession += " ";
      _accession += _accession_symbol;
    }
    else if( _header == _name ){
      extend_title(_title_name,buf);
    }
    /*
    else if( _header == _disease ){
      extend_title(_title_disease,buf);
    }
    */
    else if( _header == _orthology ){
      extend_title(_title_orthology,buf);

      // ex.)
      // KO: K04180  chemokine (C-C motif) receptor 5
      // -> K04180 chemokine (C-C motif) receptor 5
      string::size_type ret = _title_orthology.find(" KO: ");
      if( ret != _title_orthology.npos )
	_title_orthology.replace(ret,4,";");
      ret = _title_orthology.find("  ");
      while( ret != _title_orthology.npos ){
	_title_orthology.erase(ret,1);
	ret = _title_orthology.find("  ");
      }
    }
    // 配列データの処理
    // ※ この統計処理は seqnew の実行結果に出るのみ
    else if( _header == _aaseq ){
      string seq = trimLeftSpaces(buf);
      int len = seq.length();

      // このエントリのアミノ酸配列の長さ
      _aaseqlen += len;

      // データベース全体のアミノ酸配列の長さ
      _total_aaseqlen += len;

      // 最長のアミノ酸配列
      if( _max_aaseqlen < _aaseqlen )
	_max_aaseqlen = _aaseqlen;
    }
    else if( _header == _ntseq ){
      string seq = trimLeftSpaces(buf);
      int len = seq.length();

      // このエントリの塩基配列の長さ
      _ntseqlen += len;

      // データベース全体の塩基配列の長さ
      _total_ntseqlen += len;

      // 最長の塩基配列
      if( _max_ntseqlen < _ntseqlen  )
	_max_ntseqlen = _ntseqlen;

      // アミノ酸と塩基の配列の長さの合計
      if( _max_seqlen < _aaseqlen + _ntseqlen )
	_max_seqlen = _aaseqlen + _ntseqlen;
    }
  }
  // エントリ終了処理
  else if( buf.find(_delim) == 0 ){
    set_size(ifs);

    // アミノ酸配列のみ
    if( _header == _aaseq ){
      set_seq_size(buf,_aaseq_size,_aaseq_offset);

      // アミノ酸配列の長さの合計
      if( _max_seqlen < _aaseqlen )
	_max_seqlen = _aaseqlen;
    }
    else{
      set_seq_size(buf,_ntseq_size,_ntseq_offset);
    }

    writer();
  }

  // 何かのヘッダ行
  else{

    // T10000/T49000 は AASEQ のあとに COMMENT行または REFERENCE行がある
    // set_header() する前に判定して処理
    // COMMENT、REFERENCE 以外の未知のヘッダが来た場合も AASEQ 用の処理をする
    if (_header == _aaseq) {
      if (_abbr == "ag"||_abbr == "vp") {
        set_size(ifs);
        set_seq_size(buf,_aaseq_size,_aaseq_offset);
        if (_max_seqlen < _aaseqlen)
          _max_seqlen = _aaseqlen;
      }
    }

    set_header(buf);

  }

}

void Genes::writer(){

  _title = _title_orthology;

  if( _is_genes )
    if( _title.empty() )
      _title = "no KO assigned";

  // KEGG が付けた情報と元からの情報のデリミタはパイプ
  if( ! _title.empty() )
    if( ! _title_original_db.empty() || ! _title_symbol.empty() || ! _title_name.empty() )
      _title += " |";

  // Original DB があれば連結
  if( ! _title.empty() && ! _title_original_db.empty() )
    _title += " ";

  _title += _title_original_db;

  // SYMBOL があれば連結
  if( ! _title.empty() && ! _title_symbol.empty() )
    _title += " ";

  _title += _title_symbol;

  // SYMBOL と NAME のデリミタはセミコロン
  if( ! _title_symbol.empty() && ! _title_name.empty() )
    _title += ";";

  // NAME があれば連結  
  if( ! _title.empty() && ! _title_name.empty() )
    _title += " ";

  _title += _title_name;

  _aaseq_id = "_aaseq_";
  _aaseq_id += _id;
  _ntseq_id = "_ntseq_";
  _ntseq_id += _id;

  Seqnew::writer();

  if( _aaseq_offset > 0 )
    Seqnew::writer(_aaseq_id,_aaseq_offset,_aaseq_size);

  if( _ntseq_offset > 0 )
    Seqnew::writer(_ntseq_id,_ntseq_offset,_ntseq_size);

  clear();
}

void Genes::clear(){

  Seqnew::clear();

  _title_symbol = "";
  _title_name = "";
  _title_original_db = "";
  _title_disease = "";
  _title_orthology = "";
  _aaseq_id = "";
  _aaseq_offset = 0;
  _aaseq_size = 0;
  _aaseqlen = 0;
  _ntseq_id = "";
  _ntseq_offset = 0;
  _ntseq_size = 0;
  _ntseqlen = 0;

}
