
#include "dbget.h"
#include "glycan.h"

using namespace std;

Glycan::Glycan():
    _entry("ENTRY"),
    _name("NAME"),
    _comp("COMPOSITION"),
    _class("CLASS"),
    _delim("///"),
    _title_name(""),
    _title_comp(""),
    _title_class(""),
    _kcf("NODE"),
    _kcf_id(""),
    _kcf_offset(0),
    _kcf_size(0){}

Glycan::~Glycan(){}

void Glycan::parser(ifstream& ifs,
		    string& buf){

  if( buf.find(_entry) == 0 ){
    set_offset(ifs,buf);
    _header = _entry;
    buf = buf.substr(_header.length());
    set_id(buf);
  }

  else if( buf.find(_name) == 0 ){
    _header = _name;
    buf = buf.substr(_header.length());
    _title_name = trimLeftSpaces(buf);
  }

  else if( buf.find(_comp) == 0 ){
    _header = _comp;
    buf = buf.substr(_header.length());
    _title_comp = trimLeftSpaces(buf);
  }

  else if( buf.find(_class) == 0 ){
    _header = _class;
    buf = buf.substr(_header.length());
    _title_class = trimLeftSpaces(buf);
  }

  // NODE行が KCF データの開始位置
  else if( buf.find(_kcf) == 0 ){
    _header = _kcf;
    _kcf_offset = ifs.tellg();
    _kcf_offset -= buf.length();
    --_kcf_offset;
  }

  // 何かの継続行
  else if( buf.find("   ") == 0 ){
    if( _header == _name ){
      extend_title(_title_name,buf);
    }
    else if( _header == _comp ){
      extend_title(_title_comp,buf);
    }
    else if( _header == _class ){
      extend_title(_title_class,buf);
    }
  }

  // エントリ終了処理
  else if( buf.find(_delim) == 0 ){
    set_size(ifs);
    _kcf_size = _size - ( _kcf_offset - _offset );
    writer();
  }

  // 何かのヘッダ行
  else{
    set_header(buf);
  }

}

void Glycan::writer(){

  _title = _title_name;

  if( ! _title.empty() && ! _title_comp.empty() )
    _title += "; ";

  _title += _title_comp;

  if( ! _title.empty() && ! _title_class.empty() )
    _title += "; ";

  _title += _title_class;

  _kcf_id = "_kcf_";
  _kcf_id += _id;

  Seqnew::writer();

  if( _kcf_offset > 0 )
    Seqnew::writer(_kcf_id,_kcf_offset,_kcf_size);

  clear();
}

void Glycan::clear(){

  Seqnew::clear();

  _title_name = "";
  _title_comp = "";
  _title_class = "";
  _kcf_id = "";
  _kcf_offset = 0;
  _kcf_size = 0;

}
