
#include "dbget.h"
#include "ncbi_cdd.h"

using namespace std;

Ncbi_cdd::Ncbi_cdd():
    _entry("ENTRY"),
    _ac("ACCESSION"),
    _name("NAME"),
    _def("DEFINITION"),
    _delim("//"),
    _title_ac(""),
    _title_name(""),
    _title_def(""),
    _aaseq("SEQUENCE"),
    _aaseq_id(""),
    _aaseq_offset(0),
    _aaseq_size(0),
    _aaseqlen(0){}

Ncbi_cdd::~Ncbi_cdd(){}

void Ncbi_cdd::parser(ifstream& ifs,
		   string& buf){

  if( buf.find(_entry) == 0 ){
    set_offset(ifs,buf);
    _header = _entry;
    buf = buf.substr(_header.length());
    set_id(buf);
  }

  else if( buf.find(_ac) == 0 ){
    _header = _ac;
    buf = buf.substr(_header.length());
    _title_ac = trimLeftSpaces(buf);

    if (!_accession.empty())
      _accession += " ";
    _accession += _title_ac;

  }

  else if( buf.find(_name) == 0 ){
    _header = _name;
    buf = buf.substr(_header.length());
    _title_name = trimLeftSpaces(buf);
  }

  else if( buf.find(_def) == 0 ){
    _header = _def;
    buf = buf.substr(_header.length());
    _title_def = trimLeftSpaces(buf);
  }

  else if( buf.find(_aaseq) == 0 ){
    _header = _aaseq;
    _aaseq_offset = ifs.tellg();
  }

  // 何かの継続行
  else if( buf.find(" ") == 0 ){
    if( _header == _def ){
      extend_title(_title_def,buf);
    }

    // 配列データの処理
    // ※ この統計処理は seqnew の実行結果に出るのみ
    else if( _header == _aaseq ){
      trimSpaces(buf);
      int len = buf.length();

      // このエントリのアミノ酸配列の長さ
      _aaseqlen += len;

      // データベース全体のアミノ酸配列の長さ
      _total_aaseqlen += len;

      // 最長のアミノ酸配列
      if( _max_aaseqlen < _aaseqlen )
	_max_aaseqlen = _aaseqlen;

      // 最長の配列(GENESならアミノ酸と塩基の配列の長さの合計)
      _max_seqlen = _max_aaseqlen;

    }
  }

  // エントリ終了処理
  else if( buf.find(_delim) == 0 ){
    set_size(ifs);

    // ここまでが SEQUENCE行なら、ここで配列データの統計情報をセット
    if( _header == _aaseq )
      set_seq_size(buf,_aaseq_size,_aaseq_offset);

    writer();
  }

  // 何かのヘッダ行
  else{

    // ここまでが SEQUENCE行なら、ここで配列データの統計情報をセット
    if( _header == _aaseq ){
      _size = ifs.tellg();
      _size -= _offset;
      set_seq_size(buf,_aaseq_size,_aaseq_offset);
    }

    set_header(buf);
  }

}

void Ncbi_cdd::writer(){

  // プライマリ accession は角括弧で括る
  if (!_title_ac.empty()) {
    _title = "[";
    _title += _title_ac;
    _title += "] ";
  }

  // name はカンマで連結
  if (!_title_name.empty()) {
    _title += _title_name;
    _title += ", ";
  }
    
  _title += _title_def;

  _aaseq_id = "_aaseq_";
  _aaseq_id += _id;

  Seqnew::writer();

  if( _aaseq_offset > 0 )
    Seqnew::writer(_aaseq_id,_aaseq_offset,_aaseq_size);

  clear();
}

void Ncbi_cdd::clear(){

  Seqnew::clear();

  _title_ac = "";
  _title_name = "";
  _title_def = "";
  _aaseq_id = "";
  _aaseq_offset = 0;
  _aaseq_size = 0;
  _aaseqlen = 0;

}
