
#include "dbget.h"
#include "netdbget.h"

bool netdbget(const string& server,
	      const int& port,
	      const string& program,
	      const list<string>& commandline,
	      const string& proxy)
{
  return netdbget(server,port,program,commandline,proxy,cout);
}

bool netdbget(const string& server,
	      const int& port,
	      const string& program,
	      const list<string>& commandline,
	      const string& proxy,
	      ostream& ofs)
{

  bool flg = false;

  if( server.find("http://") != server.npos ){
    httpClient *sock;

    // http://sample.server.com/cgi-bin/cgi.pl?
    string webserver = server.substr(7); // sample.server.com/cgi-bin/cgi.pl?
    string host;
    string path;
    string::size_type slash = webserver.find("/");
    if( slash != webserver.npos ){
      host = webserver.substr(0,slash); // sample.server.com
      path = webserver.substr(slash); // /cgi-bin/cgi.pl?
    }else{
      host = webserver;
      path = "/";
    }

    // /cgi-bin/cgi.pl?bfind+arg1+arg2+...
    path += program;

    string::size_type rest = host.find("rest");
    int cnt = 0;
    lci citr = commandline.begin();
    while( citr != commandline.end() ){
      if( rest != host.npos && rest == 0 && cnt == 0 )
	path += "/";
      else if( rest != host.npos && rest == 0 && cnt == 1 && program == "find" )
	path += "/";
      else
	path += "+";
      path += *citr++;
      ++cnt;
    }

    if( ! proxy.empty() ){
      // proxy == proxy http://proxy.server.com 8080
      vector<string> proxyvec;
      splitToList(proxyvec,proxy," ");

      vci pitr = proxyvec.begin();
      string proxyserver = *(++pitr); // http://proxy.server.com
      string proxypnum = *(++pitr);       // 8080
      int proxyport = mystoi(proxypnum); // 8080

      if( proxyserver.find("http://") != proxyserver.npos )
	proxyserver = proxyserver.substr(7); // proxy.server.com

      if( port != 80 ){
	host += ":";
	host += itos(port); // sample.server.com:81
      }

      // proxy経由でコネクションを開く
      sock = new httpClient(proxyserver,(unsigned short)proxyport,host,path);
    }
    else{
      // コネクションを開く
      sock = new httpClient(host,(unsigned short)port,path);
    }

    if( sock->dbget_connect() )
      if( sock->dbget_send() )
	if( sock->dbget_receive() )
	  flg = true;

  }else{
    string line;
    tcpClient *sock;
    sock = new tcpClient(server,(unsigned short)port);

    if( sock->dbget_connect() ){
      // ヘッダメッセージを受信する
      line = sock->dbget_receive_line();

      while( !line.empty() ){
	if( line.substr(0,14) == "*Request-IDent" )
	  break;
	line = sock->dbget_receive_line();
      }

      // コマンドを送信する
      string message;
      lci citr = commandline.begin();
      while( citr != commandline.end() ){
	message += " ";
	message += *citr++;
      }
      message += "\r\n";

      if( sock->dbget_send(program + message) )
	flg = sock->dbget_receive();

    }
    delete sock;
  }

  return flg;

}

bool tcpClient::dbget_connect(){
  struct hostent *hostinfo;
  struct sockaddr_in address;

  // ホスト情報取得処理
  hostinfo = gethostbyname(server.c_str());

  // 情報が見つからない場合のエラー処理
  if(hostinfo == NULL){
    cerr << server << " not found." << endl;
    return false;
  }

  // sockaddr_in 構造体をゼロクリア
  memset((char *)&address,0,sizeof(address));

  // sockaddr_in 構造体型の変数をセット
  address.sin_family = AF_INET;   // アドレスファミリー
  address.sin_port = htons(port); // port番号

  // IPアドレス
  memcpy((char *)&address.sin_addr,(char *)hostinfo->h_addr,hostinfo->h_length);

  // クライアント用ソケット作成
  if ( ( sd = socket(AF_INET, SOCK_STREAM, 0) ) < 0 ){
    fprintf(stderr, "socket() failed.\n");
    return false;
  }

  // サーバに接続
  if ( connect(sd, (struct sockaddr *)&address, sizeof(address)) == -1 ){
    fprintf(stderr, "connect() failed.\n");
    return false;
  }

  // FILE 構造体を作成
  fp = fdopen(sd, "r+");
  if ( fp == NULL ){
    fprintf(stderr, "fdopen() failed.\n");
    return false;
  }

  // バッファリング OFF
  if( setvbuf(fp, NULL, _IONBF, 0) != 0 )
    return false;

  return true;

}

bool tcpClient::dbget_send(const string& command){
  if( write(sd, command.c_str(), command.length()) < 0 )
    return false;
  return true;
}

string tcpClient::dbget_receive_line(){
  int result;
  fd_set readfds;
  struct timeval timeout;

  FD_ZERO(&readfds);
  FD_SET(sd, &readfds);

  timeout.tv_sec = 600;
  timeout.tv_usec = 0;

  // サーバーからのデータを読む
  char buf[buffersize+1];
  memset(buf,'\0',sizeof(buf));
  unsigned cnt;
  string buffer = "";

  for(cnt = 0; cnt < buffersize; cnt++){

    // 読めるまで待ち合わせる
    // Cygwin でも Cygwin64 でも select() に渡せるファイル記述子の最大値は 64
    // (cout << FD_SETSIZE << endl;)
    // (sys/types.h のインクルード前に FD_SETSIZE で変更可能)
    result = select(sd+1, &readfds, (fd_set *)0, (fd_set *)0, &timeout);

    if(result == 0){
      printf("buf = %s\n",buf);
      cerr << "Server timeout" << endl;
      break;
    }

    if(result == -1){
      cerr << "Select failed" << endl;
      break;
    }

    // 1バイトずつ改行コードまで読む
    result = read(sd, &buf[cnt], 1);

    if(result<0){
      cerr << "Socket read error" << endl;
      return "";
    }else if( result == 0 ){
      return "";
    }else{
      if(buf[cnt] == '\n'){
	buf[cnt] = '\0';
	break;
      }
    }
  }

  if(cnt == buffersize){
    cerr << "Too long line" << endl;
  }

  buffer = buf;
  return buffer;

}

bool tcpClient::dbget_receive()
{
  char buf[buffersize+1];
  memset(buf,'\0',sizeof(buf));

  while(1){
    int n = read(sd,buf,buffersize);
    if( n == 0 )
      break;
    if( write(1,buf,n) < 0 )
      break;
  }

  return true;
}

bool httpClient::dbget_send(){
  string version = VERSION;
  int reqlen = 0;
  int bufflen = 256;

  reqlen = host.length() + path.length();
  if ( reqlen > bufflen-1 ){
    fprintf(stderr, "Too long message.\n");
    return false;
  }

  // サーバに送信
  if( ! proxy.empty() ){
    fprintf(fp, "GET http://%s%s HTTP/1.1\r\n", host.c_str(), path.c_str());
    fprintf(fp, "Host: %s\r\n", host.c_str());
  }else{
    if( port == 80 ){
      fprintf(fp, "GET http://%s%s HTTP/1.1\r\n", host.c_str(), path.c_str());
      fprintf(fp, "Host: %s\r\n", host.c_str());
    }else{
      fprintf(fp, "GET http://%s:%d%s HTTP/1.1\r\n", host.c_str(), port, path.c_str());
      fprintf(fp, "Host: %s:%d\r\n", host.c_str(), port);
    }
  }
  fprintf(fp, "User-Agent: DBGET/%s\r\n",version.c_str());
  fprintf(fp, "Accept: text/xml,application/xml,application/xhtml+xml,text/html,text/plain\r\n");
  fprintf(fp, "Accept-Charset: iso-8859-1\r\n");
  fprintf(fp, "Connection: close\r\n");
  fprintf(fp, "Referer: %s\r\n",host.c_str());

  fprintf(fp, "\r\n");

  return true;
}

// 受信して表示 バッファリングなし
// HTTP/1.0 はチャンク形式なし
// HTTP/1.1 のチャンク形式対応
bool httpClient::dbget_receive(){
  char buf[buffersize+1];
  memset(buf,'\0',sizeof(buf));
  int n = 0;

  while(1){

    // メッセージ本体
    if( bodyflag ){

      if( chunk ){

	// メッセージ本体の直前に16進でチャンクサイズが出る
	if ( fgets(buf, sizeof(buf), fp) == NULL )
	  break;

	// メッセージ本体の直後に空行が出る
	if( strncmp(buf,"\r\n",2) == 0 )
	  continue;

	int linelength = strlen(buf)-2;
	buf[linelength] = '\0'; // \r\n を落す
	chunksize = strtol(buf,NULL,16); // 文字列を16進数に変換

	// チャンク形式エンコーディングは、サイズが 0 のチャンクで終り
	if( chunksize == 0 ) // && strncmp(buf,"0",1) == 0 )
	  break;

	unsigned r = 0;         // チャンクから読み込み済みのサイズ
	unsigned s = chunksize; // チャンクから読み込むサイズ
	unsigned t = chunksize;
	while( r < t ){
	  if( buffersize < s )
	    s = buffersize;
	  n = read(sd,buf,s);
	  if( n == 0 )
	    break;
	  if( write(1,buf,n) < 0 )
	    break;
	  r += n;
	  s = t - r;
	}

      }
      else{

	n = read(sd,buf,buffersize);
	if( n == 0 )
	  break;
	if( write(1,buf,n) < 0 )
	  break;

      }

      count = true;
      continue;

    }

    // レスポンスヘッダ解析
    if ( fgets(buf, sizeof(buf), fp) == NULL )
      break;

    // レスポンスヘッダの終り
    if( strncmp(buf,"\r\n",2) == 0 ){
      bodyflag = 1;
    }
    else{ // レスポンスヘッダ
      // チャンク形式エンコーディングが指定された
      if( strcasecmp(buf,"Transfer-Encoding: chunked\r\n") == 0 )
	chunk = 1;
    }

  } // while(1)

  return count;

}



