
#ifndef __NETDBGET__
#define __NETDBGET__

#include <iostream>
#include <fstream>
#include <string>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/select.h>

using namespace std;

bool netdbget(const string&, // url or server
	      const int&,    // port
	      const string&, // program(bfind or bget ...)
	      const list<string>&, // command line arguments
	      const string&); // proxy

bool netdbget(const string&, // url or server
	      const int&,    // port
	      const string&, // program(bfind or bget ...)
	      const list<string>&, // command line arguments
	      const string&, // proxy
	      ostream& );    // output stream

class tcpClient {

public :

  string server;
  unsigned short port;
  int sd; // socket handler
  FILE *fp;
  unsigned buffersize;

  tcpClient(){}
  tcpClient(const string& aserv,
	    const unsigned short aport){
    server = aserv;
    port = aport;
    buffersize = MAXSIZE;
  }
  virtual ~tcpClient(){
    fclose(fp);
    close(sd);
  }

  virtual bool dbget_connect();
  bool dbget_send(const string&);
  string dbget_receive_line();
  virtual bool dbget_receive();

};

class httpClient : public tcpClient {

  string host;
  string path;
  string proxy;

  // for receive()
  int  bodyflag;
  int  chunk;
  int  chunksize;
  bool count;

public :

  httpClient(){}
  // proxyなし
  httpClient(const string& aserv,
	     const unsigned short aport,
	     const string& apath){
    server = aserv;
    port = aport;
    host = aserv;
    path = apath;
    buffersize = MAXSIZE;
    bodyflag = 0;
    chunk = 0;
    chunksize = 0;
    count = false;
  }
  // proxyあり
  httpClient(const string& aserv,
	     const unsigned short aport,
	     const string& ahost,
	     const string& apath){
    proxy = aserv;
    server = aserv;
    port = aport;
    host = ahost;
    path = apath;
    buffersize = MAXSIZE;
    bodyflag = 0;
    chunk = 0;
    chunksize = 0;
    count = false;
  }
  ~httpClient(){
    fclose(fp);
    close(sd);
  }

  bool dbget_send();
  bool dbget_receive();

};

#endif
