
#include "dbget.h"
#include "pdb.h"

using namespace std;

Pdb::Pdb():
    _entry("HEADER"),
    _tit("TITLE"),
    _compnd("COMPND"),
    _delim("END"),
    _title_tit(""),
    _title_compnd(""),
    _aaseq("SEQRES"),
    _aaseq_id(""),
    _aaseq_offset(0),
    _aaseq_size(0),
    _aaseqlen(0){}

Pdb::~Pdb(){}

void Pdb::parser(ifstream& ifs,
		  string& buf){

  if( buf.find(_entry) == 0 ){
    set_offset(ifs,buf);
    _header = _entry;

    // _id のセットは特殊処理 - 62カラム目以降の最初の単語
    // ex.)
    // HEADER    TEXTURE OF CONNECTIVE TISSUE      23-MAY-78   1AGA
    buf = buf.substr(62);
    set_id(buf);

  }

  else if( buf.find(_tit) == 0 ){
    _header = _tit;

    // 10カラム目以降
    // ex.)
    // TITLE     CRYSTAL STRUCTURE OF BACILLUS SUBTILIS CODW, A NON-                   
    // TITLE    2 CANONICAL HSLV-LIKE PEPTIDASE WITH AN IMPAIRED CATALYTIC            
    buf = buf.substr(10);

    // 行の右端がスペースパディング
    buf = trimRightSpaces(buf);

    if( ! _title_tit.empty() )
      extend_title(_title_tit,buf);
    else
      _title_tit = trimLeftSpaces(buf);
  }

  else if( buf.find(_compnd) == 0 ){
    _header = _compnd;

    // 10カラム目以降
    // ex.)
    // COMPND    MOL_ID: 1;                                                            
    // COMPND   2 MOLECULE: DNAJ;                                                      
    buf = buf.substr(10);

    // 行の右端がスペースパディング
    buf = trimRightSpaces(buf);

    if( ! _title_compnd.empty() )
      extend_title(_title_compnd,buf);
    else
      _title_compnd = trimLeftSpaces(buf);
  }

  else if( buf.find(_aaseq) == 0 ){
    // SEQRES行が配列データ
    if( _header != _aaseq ){
      _aaseq_offset = ifs.tellg();
      _aaseq_offset -= buf.length();
      --_aaseq_offset;
    }

    _header = _aaseq;

    // 配列データの処理
    // ※ この統計処理は seqnew の実行結果に出るのみ
    // 19カラム目以降
    // ex.)
    // SEQRES   1 A  103  ALA LYS GLN ASP TYR TYR GLU ILE LEU GLY VAL SER LYS
    buf = buf.substr(19);
    trimSpaces(buf);
    int len = buf.length() / 3;

    // このエントリのアミノ酸配列の長さ
    _aaseqlen += len;

    // データベース全体のアミノ酸配列の長さ
    _total_aaseqlen += len;

    // 最長のアミノ酸配列
    if( _max_aaseqlen < _aaseqlen )
      _max_aaseqlen = _aaseqlen;

    // 最長の配列(GENESならアミノ酸と塩基の配列の長さの合計)
    _max_seqlen = _max_aaseqlen;

  }


  // エントリ終了処理
  // 特殊処理 - ENDMDL で始まる行がある
  else if( buf.find(_delim) == 0 && buf.find("ENDMDL") != 0 ){
    set_size(ifs);

    // ここまでが SEQRES行なら、ここで配列データの統計情報をセット
    if( _header == _aaseq )
      set_seq_size(buf,_aaseq_size,_aaseq_offset);

    writer();
  }

  // 何かのヘッダ行
  else{
    // ここまでが SEQRES行なら、ここで配列データの統計情報をセット
    if( _header == _aaseq ){
      _size = ifs.tellg();
      _size -= _offset;
      set_seq_size(buf,_aaseq_size,_aaseq_offset);
    }

    set_header(buf);
  }

}

void Pdb::writer(){

  _title = _title_tit;

  // TITLE 行がない場合に限り COMPND行のデータをタイトルデータとする
  // TITLE 行がある場合は COMPND行のデータはタイトルデータとしない
  if( _title.empty() && ! _title_compnd.empty() )
    _title = _title_compnd;

  _aaseq_id = "_aaseq_";
  _aaseq_id += _id;

  Seqnew::writer();

  if( _aaseq_offset > 0 )
    Seqnew::writer(_aaseq_id,_aaseq_offset,_aaseq_size);

  clear();
}

void Pdb::clear(){

  Seqnew::clear();

  _title_tit = "";
  _title_compnd = "";
  _aaseq_id = "";
  _aaseq_offset = 0;
  _aaseq_size = 0;
  _aaseqlen = 0;

}
