
#include "dbget.h"
#include "pfam.h"

using namespace std;

Pfam::Pfam():
    _entry("#=GF ID"),
    _ac("#=GF AC"),
    _de("#=GF DE"),
    _delim("//"),
    _title_ac(""),
    _title_de(""){}

Pfam::~Pfam(){}

void Pfam::parser(ifstream& ifs,
		   string& buf){

  if( buf.find(_entry) == 0 ){
    set_offset(ifs,buf);
    _header = _entry;
    buf = buf.substr(_header.length());
    set_id(buf);
  }

  else if( buf.find(_ac) == 0 ){
    _header = _ac;
    buf = buf.substr(_header.length());

    // accession からセミコロンを除く
    // ex.) AC   P15455; Q3E711; Q9FFH7;
    //      → P15455 Q3E711 Q9FFH7
    buf = trimLeftSpaces(buf);
    trimChars(buf,";");

    // スペースで分割
    list<string> lbuf;
    splitToList(lbuf,buf," ");
    list<string>::const_iterator itr = lbuf.begin();
    while( itr != lbuf.end() ){
      // ピリオドの後ろはバージョン番号なので accession には含めない
      // ピリオドで区切られた最初の単語を accession　にする
      string pfacc = *itr++;
      string::size_type ret = pfacc.find(".");
      if( ret != pfacc.npos )
	pfacc = pfacc.substr(0,ret);

      // スペースで連結
      if( ! _accession.empty() )
	_accession += " ";
      _accession += pfacc;

      // タイトルに使用する accession はプライマリのみ
      if( _title_ac.empty() )
	_title_ac = pfacc;
    }

  }

  else if( buf.find(_de) == 0 ){
    _header = _de;
    buf = buf.substr(_header.length());

    if( ! _title_de.empty() )
      extend_title(_title_de,buf);
    else
      _title_de = trimLeftSpaces(buf);
  }

  // エントリ終了処理
  else if( buf.find(_delim) == 0 ){
    set_size(ifs);
    writer();
  }

}

void Pfam::writer(){

  // プライマリ accession は大括弧で括る
  if( ! _title_ac.empty() ){
    _title = "[";
    _title += _title_ac;
    _title += "]";
  }

  // accession の後ろは半角スペースで連結
  if( ! _title.empty() && ! _title_de.empty() )
    _title += " ";

  _title += _title_de;

  Seqnew::writer();

  clear();
}

void Pfam::clear(){

  Seqnew::clear();

  _title_ac = "";
  _title_de = "";

}
