
#include "dbget.h"
#include "prosdoc.h"

using namespace std;

Prosdoc::Prosdoc():
    _entry("{PDOC"),
    _de("* "),
    _delim("{END}"),
    _title_de(""){}

Prosdoc::~Prosdoc(){}

void Prosdoc::parser(ifstream& ifs,
		     string& buf){

  if( buf.find(_entry) == 0 ){
    set_offset(ifs,buf);
    _header = _entry;
    // ex.) {PDOC00001} → PDOC00001
    buf = buf.substr(1,9);

    // フラットファイルの先頭にコメントヘッダ部がある
    // コメントヘッダ部のデリミタはエントリデリミタと同じ
    // この部分は何もせずスキップする
    if( buf != "PDOC00000" )
      set_id(buf);
  }

  else if( buf.find(_de) == 0 ){
    _header = _de;
    buf = buf.substr(_header.length());

    // ex.) * N-glycosylation site *
    //      → N-glycosylation site
    string::size_type ret = buf.find(" *");
    if( ret != buf.npos )
      buf = buf.substr(0,ret);

    if( ! _title_de.empty() )
      extend_title(_title_de,buf);
    else
      _title_de = trimLeftSpaces(buf);
  }

  // エントリ終了処理
  else if( buf.find(_delim) == 0 ){
    // フラットファイルの先頭にコメントヘッダ部がある
    // コメントヘッダ部のデリミタはエントリデリミタと同じ
    // この部分は何もせずスキップする
    if( ! _id.empty() ){
      set_size(ifs);
      writer();
    }
    else
      clear();
  }

}

void Prosdoc::writer(){

  // プライマリ accession は大括弧で括る
  if( ! _title_ac.empty() ){
    _title = "[";
    _title += _title_ac;
    _title += "]";
  }

  // accession の後ろは半角スペースで連結
  if( ! _title.empty() && ! _title_de.empty() )
    _title += " ";

  _title += _title_de;

  Seqnew::writer();

  clear();
}

void Prosdoc::clear(){

  Seqnew::clear();

  _title_ac = "";
  _title_de = "";

}
