
#include "dbget.h"
#include "prosite.h"

using namespace std;

Prosite::Prosite():
    _entry("ID "),
    _ac("AC "),
    _de("DE "),
    _delim("//"),
    _title_ac(""),
    _title_de(""){}

Prosite::~Prosite(){}

void Prosite::parser(ifstream& ifs,
		     string& buf){

  if( buf.find(_entry) == 0 ){
    set_offset(ifs,buf);
    _header = _entry;
    buf = buf.substr(_header.length());

    // ID からセミコロンを除く
    // ex.) GREAB_1; PATTERN.
    //      → GREAB_1 PATTERN.
    buf = trimLeftSpaces(buf);
    trimChars(buf,";");

    set_id(buf);
  }

  else if( buf.find(_ac) == 0 ){
    _header = _ac;
    buf = buf.substr(_header.length());

    // accession からセミコロンを除く
    // ex.) AC   P15455; Q3E711; Q9FFH7;
    //      → P15455 Q3E711 Q9FFH7
    buf = trimLeftSpaces(buf);
    trimChars(buf,";");

    // スペースで連結
    if( ! _accession.empty() )
      _accession += " ";
    _accession += buf;

    // タイトルに使用する accession はプライマリのみ
    if( _title_ac.empty() ){
      // 半角スペースで区切られた最初の単語
      string::size_type ret = buf.find(" ");
      if( ret != buf.npos )
        buf = buf.substr(0,ret);
      _title_ac = buf;
    }

  }

  else if( buf.find(_de) == 0 ){
    _header = _de;
    buf = buf.substr(_header.length());

    if( ! _title_de.empty() )
      extend_title(_title_de,buf);
    else
      _title_de = trimLeftSpaces(buf);
  }

  // エントリ終了処理
  else if( buf.find(_delim) == 0 ){
    // フラットファイルの先頭にコメントヘッダ部がある
    // コメントヘッダ部のデリミタはエントリデリミタと同じ
    // この部分は何もせずスキップする
    if( ! _id.empty() ){
      set_size(ifs);
      writer();
    }
    else
      clear();

  }

}

void Prosite::writer(){

  // プライマリ accession は大括弧で括る
  if( ! _title_ac.empty() ){
    _title = "[";
    _title += _title_ac;
    _title += "]";
  }

  // accession の後ろは半角スペースで連結
  if( ! _title.empty() && ! _title_de.empty() )
    _title += " ";

  _title += _title_de;

  Seqnew::writer();

  clear();
}

void Prosite::clear(){

  Seqnew::clear();

  _title_ac = "";
  _title_de = "";

}
