
#include "dbget.h"
#include "readdbtab.h"

// dbtab の情報を取得
bool readFile(const string& dbtab,
	      const string& db,
	      list<string>& inputlist,
	      string& proxy)
{
  // 初期化
  inputlist.clear();

  // dbtab.cdb を使う
  if( readDbtabCDB(dbtab,db,inputlist,proxy) )
    return true;

  // dbtab.bfind と dbtab.bfind.bin を使う -- Obsolete
  //list<string> linedata;
  //if( readDbtabBfind(dbtab,db,linedata,proxy) )
  //  if( readDbtabBfindBin(dbtab,linedata,inputlist) )
  //    return true;

  // dbtab.bin を使う -- Obsolete
  //if( readDbtabBin(dbtab,db,inputlist,proxy) )
  //  return true;

  // dbtab を使う
  // dbset は dbtab の情報のうち、リンク情報以外の情報を持つ
  // linkinfo は各データベースのリンク情報を持つ
  // namelist は dbtab に記載されるデータベースの順番を保持する
  // ここでは dbset 以外は使わない
  multimap<string,BasicInfo> dbset;
  multimap<string,string> linkinfo;
  list<string> namelist;
  if( readDbtab(dbtab,dbset,linkinfo,namelist) ){

    string dbname = db;
    if( isTNumber(dbname) || isRGNumber(dbname) )
      Upper(dbname);
    else
      Lower(dbname);

    // BasicInfo を得る
    pair<mci,mci> mp = dbset.equal_range(dbname);

    // dbが見つからなかった
    //if( distance(mp.first,mp.second) == 0 )
    //  return false;

    mci mitr = mp.first; // dbsetの反復子

    // dbが見つからなかった
    if( mitr == mp.second )
      return false;

    BasicInfo info = mitr->second;

    string line;
    while(mitr != mp.second){
      info = mitr->second;
      // local database
      if( info.getPath().length() > 0 ){
	line = info.getName() + " "
	  + info.getType() + " "
	  + info.getPath() + " "
	  + info.getNick();
      }
      // netdbget database
      else if( info.getHost().length() > 0 ){
	int port = info.getPort();
	string pnum = itos(port);
	line = info.getName() + " "
	  + info.getHost() + " "
	  + pnum;
      }
      inputlist.push_back(line);
      mitr++;
    }

    // proxyサーバー
    mp = dbset.equal_range("proxy");
    mitr = mp.first;
    while(mitr != mp.second){
      info = mitr->second;
      if( info.getHost().length() > 0 ){
	int port = info.getPort();
	string pnum = itos(port);
	proxy = info.getName() + " "
	  + info.getHost() + " "
	  + pnum;
      }
      mitr++;
    }

    return true;

  }

  return false;

}

bool readDbtab(const string& infile,
	       multimap<string,BasicInfo>& dbset,
	       multimap<string,string>& linkinfo,
	       list<string>& namelist)
{

  ifstream ifs(infile.c_str());
  if( !ifs ){
    return false;
  }

  // parse database information for each line
  string buf;
  while( ! ifs.eof() ){
    getline(ifs,buf);
    if( buf[0] == '#' ) continue;
    if( buf.length() == 0 ) continue;
    if( buf.length() > 4*BUFSIZ ){
      cerr << buf << endl;
      cerr << "Too long line in " << infile << endl;
      cerr << "Line length: " << buf.length() << endl;
      continue;
    }

    // $BIOROOT -> /bio
    expandMacro(buf);

    // check each character to split line with space or tab
    int pos = 0;
    int len = buf.length();
    list<string> linedata;
    for( int i = 0; i < len; i++ ){
      if( buf[i] == ' ' || buf[i] == '\t' ){
	if( pos != i ){
	  // get an element
	  linedata.push_back(buf.substr(pos,i-pos));
	}
	pos = i+1;
      }
    }
    // get final element
    linedata.push_back(buf.substr(pos,buf.length()-pos));

    // at least 3 elements are necessary for each line
    if( linedata.size() < 3 ){
      cerr << "Undefined line: " << buf << endl;
      continue;
    }

    // parse line data
    lci itr = linedata.begin();

    string name = *itr++;
    string type = *itr++;
    string path = *itr++;
    string nick = name;
    string host;
    int port = DBGET_SERVER_PORT;

    if( isTNumber(name) || isRGNumber(name) )
      Upper(name);
    else
      Lower(name);

    // unique name check
    pair<mci,mci> mp = dbset.equal_range(name);
    if( mp.first != mp.second ){
      cerr << name << " is already defined." << endl;
      continue;
    }

    string pnum = itos(port);
    string link = name;
    Lower(type);

    // for netdbget dbget
    if( path[0] == '@' || type == "netdbget" ){
      type = "netdbget";
      if( path[0] == '@' ) path.erase(0,1);
      // path is dbget.genome.ad.jp:3266
      string::size_type ret = path.find(":");
      if( ret != path.npos && ret < path.length() ){
	host = path.substr(0,ret);
	if( ret + 1 < path.length() ){
	  pnum = path.substr(ret+1,path.length()-ret);
	  port = mystoi(pnum); // string to int
	}
      }else{
	host = path;
      }
    }
    if( type == "url" ){
      string protocol = path.substr(0,7);
      type = "netdbget";
      pnum = "80";
      port = 80;
      path.erase(0,7);
      // path is www.genome.jp/dbget-bin/www_bget
      // or www.sample.com:8181/
      // or www.sample.com:8181/cgi-bin/cgi
      // or proxy.sample.com:8080
      string::size_type ret = path.find(":");
      if( ret != path.npos && ret < path.length() ){
	host = path.substr(0,ret);
	if( ret + 1 < path.length() ){
	  pnum = path.substr(ret+1,path.length()-ret);

	  ret = pnum.find("/");
	  if( ret != pnum.npos && ret < pnum.length() ){
	    host += pnum.substr(ret,pnum.length()-ret);
	    pnum.erase(ret,pnum.length()-ret);
	  }

	  port = mystoi(pnum); // string to int
	}
      }else{
	host = path;
      }
      host = protocol + host;
    }
    if ( name == "*" && type != "netdbget" ){
      cerr << "Undefined line: " << buf << endl;
      continue;
    }

    // 4th element
    // ex.) sp,dbkey=swissprot
    string str4;
    list<string> elem4;
    if( itr != linedata.end() ){
      str4 = *itr;
      splitToList(elem4,str4,",");
    }

    // parse 4th element
    int invalid_line = 0;
    int dbkey_flag = 0;
    lci itr4 = elem4.begin();
    while( itr4 != elem4.end() ){
      // 5th element derived
      string str5 = *itr4;
      string::size_type ret5 = str5.find("=");
      if( ret5 != str5.npos ){
	if( ret5 + 1 < str5.length() ){
	  string key = str5.substr(0,ret5);
	  if( isTNumber(key) || isRGNumber(key) )
	    Upper(key);
	  else
	    Lower(key);
	  if( key == "dbkey" ){
	    dbkey_flag = 1;
	    link = str5.substr(ret5+1,str5.length()-ret5);
	  }else if( key == "name" ){
	    nick = str5.substr(ret5+1,str5.length()-ret5);
	  }else{
	    cerr << "Invalid line " << buf << endl;
	    invalid_line = 1;
	    break;
	  }
	}else{
	  cerr << "Invalid line " << buf << endl;
	  invalid_line = 1;
	  break;
	}
      }else{
	nick = str5;
      }
      itr4++;
    } // parse 4th element

    // Invalid line is detected.
    if( invalid_line ) continue;

    if( type == "include" ){
      readDbtab(path,dbset,linkinfo,namelist);
    }
    else if( type == "alias" ){
      //ex.)
      //animals     alias  hsa+mmu+rno+dre+dme+cel
      //eukaryotes  alias  animals+plants+fungi+protists
      list<string> aliases;
      splitToList(aliases,path,"+");

      int no_such_db = 0;
      int wflag = 0; // flag for non-redundant name list
      lci aitr = aliases.begin();
      while( aitr != aliases.end() ){
	string alias = *aitr;
	if( isTNumber(alias) || isRGNumber(alias) )
	  Upper(alias);
	else
	  Lower(alias);

	// skip same name
	// ex.) H.sapiens alias h.sapiens
	if( name == alias ){
	  aitr++;
	  continue;
	}
	wflag = 1;

	// expand alias for dbset
	pair<mci,mci> mp = dbset.equal_range(alias);
	if( mp.first == mp.second ){
	  cerr << "No such DB " << *aitr << endl;
	  no_such_db = 1;
	  break;
	}

	// expand aliases
	multimap<string,BasicInfo> dbset_temp; // buffer for expansion
	for( mci mitr = mp.first; mitr != mp.second; mitr++ ){
	  dbset_temp.insert(make_pair(name,mitr->second));
	}

	// insert buffer to dbset
	mci titr = dbset_temp.begin();
	while( titr != dbset_temp.end() ){
	  dbset.insert(make_pair(titr->first,titr->second));
	  titr++;
	}
	dbset_temp.clear();

	// expand alias for linkinfo
	pair<sci,sci> sp = linkinfo.equal_range(alias);
	if( sp.first != sp.second && // linkinfo is optional
	    dbkey_flag == 0 ){       // linkinfo is inherited
	  // expand aliases
	  multimap<string,string> linkinfo_temp; // buffer for expansion
	  for( sci sitr = sp.first; sitr != sp.second; sitr++ ){
	    linkinfo_temp.insert(make_pair(name,sitr->second));
	  }

	  // insert buffer to linkinfo
	  sci iitr = linkinfo_temp.begin();
	  while( iitr != linkinfo_temp.end() ){
	    linkinfo.insert(make_pair(iitr->first,iitr->second));
	    iitr++;
	  }
	  linkinfo_temp.clear();
	}

	aitr++;
      } // while aliases

      // No such DB is detected.
      if( no_such_db ) continue;

      // wflag == 1 means non-redundant name
      // old btab command is redundant
      // ex.) H.sapiens alias h.sapiens
      if( wflag ){
	if( dbkey_flag != 0 ){ // linkinfo is overridden
	  linkinfo.insert(make_pair(name,link));
	}
	namelist.push_back(name);
      }
    }
    else if( type == "netdbget" || type == "url" ){
      dbset.insert(make_pair(name,BasicInfo(name,type,host,port)));
      // link info is same as netdbget info
      link = name;
      link += " ";
      link += host;
      link += " ";
      link += pnum;
      linkinfo.insert(make_pair(name,link));
      namelist.push_back(name);
    }
    else if( type == "external" ){
      dbset.insert(make_pair(name,BasicInfo()));
      linkinfo.insert(make_pair(name,link));
      namelist.push_back(name);
    }
    else{
      // basic information
      dbset.insert(make_pair(name,BasicInfo(name,type,path,nick)));
      linkinfo.insert(make_pair(name,link));
      namelist.push_back(name);
    }


  } // parse each line
  ifs.close();
  return true;
}

// dbtab.cdb を使う

bool readDbtabCDB(const string& infile,
                  const string& db,
                  list<string>& inputlist,
                  string& proxy)
{
  // /bio/etc/dbtab.cdb を開く
  struct cdb cdb;
  struct mcdb m;
  struct mcdb_mmap map;
  int fd;
  bool mcdb = false;

#ifdef MCDB
  if (!open_index_mcdb(fd,&m,&map,infile,mcdb))
    if (!open_index(fd,&cdb,infile))
      return false;
#else
  if (!open_index(fd,&cdb,infile))
    if (!open_index_mcdb(fd,&m,&map,infile,mcdb))
      return false;
#endif

  bool flg = false;

  // proxyサーバーは未定義でもよいので
  // flg は false でも構わない
  string key = "proxy";
  if (mcdb) {
    flg = get_value_mcdb(&m,&map,key,proxy);
  }
  else {
    flg = get_value(&cdb,key,proxy);
  }

  string buf = "dbtab";
  key = db;
  if (mcdb) {
    flg = get_value_mcdb(&m,&map,key,buf);
    close_index_mcdb(fd,&map);
  }
  else {
    flg = get_value(&cdb,key,buf);
    close_index(fd,&cdb);
  }

  if (!flg)
    return false;

  splitToList(inputlist,buf,"\t");
  return true;
}

// dbtab.bin を使う
bool readDbtabBin(const string& infile,
		  const string& db,
		  list<string>& inputlist,
		  string& proxy)
{
  string dbtabbin = infile + ".bin";
  ifstream ifs(dbtabbin.c_str());
  if( !ifs ){
    return false;
  }

  string dbname = db;
  Lower(dbname); // データベース名は ASCII小文字で記載されている

  string buf;
  while( ! ifs.eof() ){
    getline(ifs,buf);

    // 半角スペースで区切られた文字列のリスト
    int pos = 0;
    int len = buf.length();
    list<string> linedata;
    for( int i = 0; i < len; i++ ){
      if( buf[i] == ' ' || buf[i] == '\t' ){
	if( pos != i ){
	  // 要素を追加
	  linedata.push_back(buf.substr(pos,i-pos));
	}
	pos = i+1;
      }
    }
    // 最後の要素はリンク情報でここでは含めない
    // linedata.push_back(buf.substr(pos,buf.length()-pos));

    // 要素数は２
    if( linedata.size() == 2 ){
      // ex.)
      // genbank-nonst 105 1
      // ligand 6 6
      lci itr = linedata.begin();
      string name  = *itr++;
      string basic = *itr++;

      // proxyサーバー
      if( name == "proxy" ){
	int nbasic = mystoi(basic);
	for( int i = 0; i < nbasic; i++ ){
	  getline(ifs,buf);
	  proxy = buf;
	}
	continue;
      }

      if( name != dbname ) continue;

      int nbasic = mystoi(basic);
      for( int i = 0; i < nbasic; i++ ){
	getline(ifs,buf);
	inputlist.push_back(buf);
      }

      ifs.close();
      return true;

    }
  }

  ifs.close();
  return false; // Not found
}

// dbtab.bfind と dbtab.bfind.bin を使う
bool readDbtabBfind(const string& infile,
		    const string& db,
		    list<string>& linedata,
		    string& proxy)
{
  string dbtabbfind = infile + ".bfind";
  ifstream ifs(dbtabbfind.c_str());
  if( !ifs ){
    return false;
  }

  string dbname = db;
  Lower(dbname); // データベース名は ASCII小文字で記載されている

  string buf;
  while( ! ifs.eof() ){
    getline(ifs,buf);

    // タブ区切りを文字列のリストに変換
    splitToList(linedata,buf,"\t");

    // 要素数は３
    if( linedata.size() == 3 ){
      lci itr = linedata.begin();

      // proxyサーバー
      if( *itr == "proxy" ){
	list<string> proxydata;
	if( readDbtabBfindBin(infile,linedata,proxydata) ){
	  lci pitr = proxydata.begin();
	  proxy = *pitr;
	}
	continue;
      }

      // 先頭の要素がデータベース名
      if( *itr != dbname ) continue;

      ifs.close();
      return true;
    }
  }

  ifs.close();
  return false; // Not found
}

bool readDbtabBfindBin(const string& infile,
		       const list<string>& linedata,
		       list<string>& inputlist)
{
  // 要素数は３
  if( linedata.size() != 3 )
    return false;

  lci itr = linedata.begin();
  string dbname = *itr++;
  string offs = *itr++;
  string lins = *itr++;

  long offset = mystol(offs);
  int  lines  = mystoi(lins);

  string dbtabbfindbin = infile + ".bfind.bin";
  ifstream ifs(dbtabbfindbin.c_str());
  if( !ifs ){
    return false;
  }

  // ランダムアクセス
  ifs.seekg(offset,ios::beg);

  string buf;
  for( int i = 0; i < lines; i++ ){
    getline(ifs,buf);
    inputlist.push_back(buf);
  }

  ifs.close();
  return true;
}

// expand macro
void expandMacro(string& buf){
  string::size_type ret;
  ret = buf.find("$");
  while( ret != buf.npos && ret + 1 < buf.length() ){
    if( ret != 0 && buf[ret-1] == '\\' ){ // not expand for \$
      buf.erase(ret-1,1); // erase '\'
      ret = buf.find("$",ret);
      continue;
    }
    string macro;
    int len = buf.length();
    for( int i = ret+1; i < len; i++ ){
      // extract macro name from "$" to non-alphnum
      if( isalnum(buf[i]) ){
	macro += buf[i];
      }else{
	break;
      }
    }

    // set macro with environment variable
    string rplstr;
    char *fromenv = getenv(macro.c_str());
    if( fromenv && *fromenv ){
      rplstr = fromenv;
    }else{
      cerr << "Undefind variable : " << macro << endl;
      exit(-1);
    }
    buf.replace(ret,macro.length()+1,rplstr);
    ret = buf.find("$",ret+1);
  }
}

