
// 引数を再帰下降構文解析で逆ポーランド記法(後置記法)に変換する
// 引数で文字列や括弧の連続は .and. を補ってあるものとする
// ex.)
// A B C → A B .and. C .and.
// A .not. B →  A B .not. .and.
// A .or. .not. ( B C ) → A B C .and. .not. .or.
// .not. A ( B .or. .not. C ( D .or. E ) ) → A .not. B C .not. .or. D E .or. .and. .and.
// .not. .not. A → syntax error
// .not. ( .not. A ) → A .not. .not.

#include "recdescent.h"

using namespace std;

RecDescentParsing::RecDescentParsing():
  syntax(true){}

RecDescentParsing::~RecDescentParsing(){}

bool RecDescentParsing::nextitr(){
  if( itr == end || ++itr == end )
    return false;
  return true;
}

void RecDescentParsing::factor()
{
  if( itr == end ) return;

  // 小括弧の演算子機能を無効にしてただの文字として扱うには
  // 以下の部分をコメントアウト
  // ---------------------------------
  else if( *itr == "(" ){
    if( ! nextitr() ){
      syntax = false;
      return;
    }
    expression();
    if( itr == end ) return;
    if( *itr == ")" ){
      if( ! nextitr() ) return;
    }
    else
      syntax = false;
  }
  // ---------------------------------

  else{
    lkeys.push_back(*itr);
    if( ! nextitr() ) return;
  }
}

void RecDescentParsing::term()
{
  if( itr == end ) return;
  else if( *itr == ".not." ){
    if( ! nextitr() ){
      syntax = false;
      return;
    }
    factor();
    lkeys.push_back(".not.");
  }
  else{
    factor();
    for(;;)
      if( itr == end ) break;
      else if( *itr == ".not." ){
        if( ! nextitr() ){
	  syntax = false;
	  return;
	}
        factor();
	lkeys.push_back(".not.");
      }
      else
        break;
  }
}

void RecDescentParsing::expression()
{
  term();
  for(;;)
    if( itr == end ) break;
    else if( *itr == ".and." ){
      if( ! nextitr() ){
	syntax = false;
	return;
      }
      term();
      lkeys.push_back(".and.");
    }
    else if( *itr == ".or." ){
      if( ! nextitr() ){
	syntax = false;
	return;
      }
      term();
      lkeys.push_back(".or.");
    }
    else
      break;
}


bool RecDescentParsing::parse
(list<string>& arguments,list<string>& keys,bool debug)
{

  if( debug )
    print_data(keys);

  int nest = 0;
  list<string>::iterator aitr = arguments.begin();
  vector<string> vkeys;
  while( aitr != arguments.end() ){

    // 演算子としての小括弧は前後に半角スペースを置き、単独で使う
    // 前後に文字があるとただの文字列として検索語に含められる
    // ex.)　A ( B .or. C ) != A (B .or. C)
    // 小括弧を演算子ではなくただの文字として扱うには以下の部分は不要
    // 小括弧を演算子として無効にするには recdescent.cpp の
    // void RecDescentParsing::factor() を修正する
    // --------------------------
    if ( *aitr == "(" )
      ++nest;
    else if ( *aitr == ")" )
      --nest;
    // --------------------------

    string key = *aitr;

    // エスケープ文字の処理
    string::size_type i = 0;
    while(1){
      i = key.find("\\(",i);
      if( i == string::npos) break;
      key.erase(i,1);
      ++i;
    }
    i = 0;
    while(1){
      i = key.find("\\)",i);
      if( i == string::npos) break;
      key.erase(i,1);
      ++i;
    }

    vkeys.push_back(key);
    ++aitr;

  }

  if( nest > 0 ){
    cerr << "Too many open \"(\"." << endl;
    return false;
  }
  if( nest < 0 ){
    cerr << "Too many close \")\"." << endl;
    return false;
  }
  if( vkeys.empty() ){
    cerr << "No expression." << endl;
    return false;
  }

  if( debug )
    print_data(vkeys);

  // 引数で文字列や括弧の連続は .and. を補う
  // ex.)
  // A B C → A .and. B .and. C
  // A .not. B → A .and. .not. B
  // A .not. ( B .or. ( C D ) ) → A .and. .not. ( B .or. ( C .and. D ) )
  bool flag = false;
  for( itr = vkeys.begin(); itr != vkeys.end(); ++itr ){
    if( flag ){
      // 前項が文字列か ) であり、その後に文字列か ( か .not. なら .and. を補う
      if( *itr != ")"     &&
	  *itr != ".and." &&
	  *itr != ".or."  ){
	itr = vkeys.insert(itr,".and.");
	++itr;
      }
    }

    // 前項が文字列か ) ならフラグを立てる
    if( *itr != "("     &&
	*itr != ".not." &&
	*itr != ".and." &&
	*itr != ".or."  )
      flag = true;
    else
      flag = false;

  }

  if( debug ){
    cout << "##### " << __FILE__ << " " << __FUNCTION__ << " " << __LINE__ << endl;
    print_data(vkeys);
  }

  // 再帰下降構文解析
  itr = vkeys.begin();
  end = vkeys.end();
  for(;;){
    expression();
    if( itr != end ) syntax = false;
    if( ! nextitr() ) break;
  }

  if( debug ){
    cout << "##### " << __FILE__ << " " << __FUNCTION__ << " " << __LINE__ << endl;
    print_data(lkeys);
  }

  keys = lkeys;
  return syntax;

}

void RecDescentParsing::print_data(const list<string>& strs)
{
  list<string>::const_iterator sitr = strs.begin();
  for( sitr = strs.begin(); sitr != strs.end(); ++sitr )
    cout << *sitr << " ";
  cout << endl;
}

void RecDescentParsing::print_data(const vector<string>& strs)
{
  vector<string>::const_iterator sitr = strs.begin();
  for( sitr = strs.begin(); sitr != strs.end(); ++sitr )
    cout << *sitr << " ";
  cout << endl;
}
