
#ifndef __SEQNEW__
#define __SEQNEW__

#include <iostream>
#include <fstream>
#include <string>

#include <cstdlib>    // for getenv 
#include <vector>
#include <list>
#include <map>

#include <fcntl.h>   // for fd = open(file, O_RDWR|O_CREAT|O_TRUNC, 0644);
#include <cdb.h>

using namespace std;

template < typename T > std::string to_string( const T& n )
{
  std::ostringstream stm ;
  stm << n ;
  return stm.str() ;
}

class usage {
public:
  usage(){

    bool mcdb = false;

#ifdef MCDB
    mcdb = true;
#endif
    
    cerr << "Usage : seqnew [option] dbname filename <...>" << endl;
    cerr << "Option : " << endl;
    cerr << "  -g            Create only index files for bget and btit command." << endl;
    cerr << "  -f            Create only index files for bfind command." << endl;
    cerr << "  -p <dbtab file> " << endl;
    cerr << "                Alternative dbtab. " << endl;
    cerr << "  -t <dbtype> -d <directory> [-b <abbr>] " << endl;
    cerr << "                <dbtype> is database type." << endl;
    cerr << "                <directory> is directory where database index files are located." << endl;
    cerr << "                <abbr> is database abbreviation." << endl;
    if (mcdb) {
      cerr << "  --cdb         Create index with TinyCDB." << endl;
      cerr << "  --mcdb        Create index with mcdb.(default)" << endl;
    }
    else {
      cerr << "  --cdb         Create index with TinyCDB.(default)" << endl;
      cerr << "  --mcdb        Create index with mcdb." << endl;
    }
    cerr << "  -h            Print this message and exit." << endl;
    cerr << "  -V            Print the version number of DBGET and exit." << endl;
  }
};

class Seqnew{

 public:

  unsigned long _counter;
  unsigned long _rejected;
  int64_t _total_aaseqlen;
  int64_t _max_aaseqlen;
  int64_t _total_ntseqlen;
  int64_t _max_ntseqlen;
  int64_t _max_seqlen;

  int64_t _offset;
  int64_t _size;
  string _id;
  string _tit_id;
  string _header;
  string _title;
  string _accession;
  string _abbr;

  string _flatfile;
  string _dbname;
  string _dir;

  bool _flg_seqnew_p;
  bool _flg_tit_only;
  bool _flg_cdb_only;
  bool _is_genes;
  bool _mcdb;

  ofstream _ofs;
  ofstream _tit;
  ofstream _tit_sub;
  ofstream _acc;

  Seqnew();
  virtual ~Seqnew();

  virtual void parser(ifstream&,
		      string&);
  virtual void extend_title(string&,
			    string&);
  virtual void writer();
  virtual void writer(const string&,
		      const int64_t&,
		      const int64_t&);
  virtual void clear();
  virtual void set_is_genes(string&,string&);

  void set_offset(ifstream&, string&);
  void set_id(string&);
  void set_size(ifstream&);
  void set_seq_size(string&,int64_t&,int64_t&);
  void set_header(string&);

  unsigned long get_counter();
  unsigned long get_rejected();
  int64_t get_total_aaseqlen();
  int64_t get_max_aaseqlen();
  int64_t get_total_ntseqlen();
  int64_t get_max_ntseqlen();
  int64_t get_max_seqlen();

  void set_seqnew_p(const bool);
  void set_tit_only(const bool);
  void set_cdb_only(const bool);
  void set_mcdb(const bool);
  void set_abbr(const string&);
  string get_abbr();
  void set_flatfile(const string&);
  void set_dbname(const string&);
  string get_dbname();
  string get_first_word(const string&);
  void set_outdir(const string&);
  bool open_outfiles();
  void close_outfiles();
  bool create_cdbs();
  bool create_cdb(const string&,const string&,int);
  bool create_mcdb(const string&,const string&);

};

#endif
