
#include "dbget.h"
#include "sequences.h"

using namespace std;

Sequences::Sequences():
  // SEQUENCES はエントリのデリミタがない
  // ヘッダ行とデータ行がそれぞれ一行ずつ
  // データ行が来たら書き込み処理でも構わないが、
  // 一般に FASTA 形式はデータ行が複数行あるため
  // ヘッダ行を書き込み処理のトリガーにする
  _entry(">"),
  _title_de(""),

  // FASTA 形式のデータは bget すればエントリ自体が配列データなので、
  // bget -f の必要はないはずだが処理をする仕様となっている

  // SEQUENCES の配列データはアミノ酸配列
  // 一般に FASTA 形式の配列データはアミノ酸とは限らないが
  // 変数は aaseq で代用する
  _aaseq_id(""),
  _aaseq_offset(0),
  _aaseq_size(0),
  _aaseqlen(0){}

Sequences::~Sequences(){}

void Sequences::parser(ifstream& ifs,
		    string& buf){

  if( buf.find(_entry) == 0 ){

    // ヘッダ行を書き込み処理のトリガーにする
    // 前エントリのデータがあれば書き込み処理
    if( ! _id.empty() )
      writer();

    set_offset(ifs,buf);
    _header = _entry;
    buf = buf.substr(_header.length());

    _aaseq_offset = ifs.tellg();

    // ex.) >GCVF01000431.1.2369 Bacteria;Proteobacteria;Gammaproteobacteria;...
    //  → ID は　GCVF01000431.1.2369
    //     タイトルは　Bacteria;Proteobacteria;Gammaproteobacteria;...

    // スペースで分割
    // 最初の単語が ID
    list<string> lbuf;
    splitToList(lbuf,buf," ");
    list<string>::iterator itr = lbuf.begin();
    set_id(*itr++);

    // 最初の単語は ID なのでそれを除いてタイトルを作成
    while( itr != lbuf.end() ){
      _title_de += *itr++;
      _title_de += " ";
    }
    _title_de = trimRightSpaces(_title_de);

  }

  // ヘッダ行以外はデータ行
  else{
    set_size(ifs);
    _aaseq_size = _size - ( _aaseq_offset - _offset );

    // 配列データの処理
    // ※ この統計処理は seqnew の実行結果に出るのみ
    int64_t len = buf.length();

    // このエントリのアミノ酸配列の長さ
    _aaseqlen += len;

    // データベース全体のアミノ酸配列の長さ
    _total_aaseqlen += len;

    // 最長のアミノ酸配列
    if( _max_aaseqlen < _aaseqlen )
      _max_aaseqlen = _aaseqlen;

    // 最長の配列(GENESならアミノ酸と塩基の配列の長さの合計)
    _max_seqlen = _max_aaseqlen;

  }

}

void Sequences::writer(){

  _title += _title_de;

  _aaseq_id = "_aaseq_";
  _aaseq_id += _id;

  Seqnew::writer();

  if( _aaseq_offset > 0 )
    Seqnew::writer(_aaseq_id,_aaseq_offset,_aaseq_size);

  clear();
}

void Sequences::clear(){

  Seqnew::clear();

  _title_de = "";

  _aaseq_id ="";
  _aaseq_offset = 0;
  _aaseq_size = 0;
  _aaseqlen = 0;

}
