
#include "dbget.h"
#include "procdata.h"
#include "splitfile.h"
#include "bfind.h"

void splitfile
(list<ProcData>& plist, string& fullpath, bool buffer_search)
{

  string titlefile = fullpath;
  titlefile += ".tit";

  int fd;

#ifdef SOL
  fd = open64(titlefile.c_str(),O_RDONLY);
#else
  fd = open(titlefile.c_str(),O_RDONLY);
#endif
  if( fd < 0 ){
    cerr << "Cannot open file: " << titlefile << endl;
    return;
  }

  struct stat st;

  if ( stat( titlefile.c_str(), & st ) < 0 ) {
    cerr << "Cannot stat() file: " << titlefile << endl;
    return;
  }

  int pbufsiz = DBGET_BUFSIZ;
  int block = BLOCKSIZE;
  char pbuf[DBGET_BUFSIZ+1] = {0};

  // 並列検索の分割位置を求める
  int64_t offset = 0;
  int64_t size = 0;
  int64_t endp = 0;

  // 非分割検索はファイル全部を 1プロセスで検索する
  if( ! buffer_search ){
    ProcData pdata(fullpath,offset,st.st_size);
    plist.push_back(pdata);
    close(fd);
    return;
  }

  // ファイルを分割してできた検索ブロック数
  for ( offset = 0; offset < st.st_size; offset += size ) {

    // 検索ブロックのサイズを求める
    size  = ( ( offset + block ) > st.st_size ) ?
      st.st_size - offset : block;

    // 検索終了点を求める
    endp = offset + size;

    // ファイルの先頭から enp バイトまでポインタを移動
    if( st.st_size > endp ) {
#ifdef SOL
      if( lseek64(fd,endp,SEEK_SET) < 0 ){
#else
      if( lseek(fd,endp,SEEK_SET) < 0 ){
#endif
        cerr << "Cannot seek() file: " << titlefile << endl;
        return;
      }

      // レコードの区切りまでに検索サイズを延長
      int count = read(fd,pbuf,pbufsiz);
      if( count < 0 )
	return;
      pbuf[count] = '\0';

      if( count > 0 ){
	for (int i = 0; i < count; ++i ){
	  if ( pbuf[i] == '\n' ){
	    size += i + 1;
	    break;
	  }
	}
      }

    }

    // ファイル名、検索開始点、サイズをセット
    ProcData pdata(fullpath,offset,size);
    plist.push_back(pdata);

  }
  close(fd);

}
