
#include "dbget.h"
#include "swissprot.h"

using namespace std;

Swissprot::Swissprot():
    _entry("ID"),
    _ac("AC"),
    _de("DE"),
    _delim("//"),
    _title_ac(""),
    _title_de(""),
    _aaseq("SQ"),
    _aaseq_id(""),
    _aaseq_offset(0),
    _aaseq_size(0),
    _aaseqlen(0){}

Swissprot::~Swissprot(){}

void Swissprot::parser(ifstream& ifs,
		       string& buf){

  if( buf.find(_entry) == 0 ){
    set_offset(ifs,buf);
    _header = _entry;
    buf = buf.substr(_header.length());
    set_id(buf);
  }

  else if( buf.find(_ac) == 0 ){
    _header = _ac;
    buf = buf.substr(_header.length());

    // accession からセミコロンを除く
    // ex.) AC   P15455; Q3E711; Q9FFH7;
    //      → P15455 Q3E711 Q9FFH7
    buf = trimLeftSpaces(buf);
    trimChars(buf,";");

    // スペースで連結
    if( ! _accession.empty() )
      _accession += " ";
    _accession += buf;

    // タイトルに使用する accession はプライマリのみ
    if( _title_ac.empty() ){
      // 半角スペースで区切られた最初の単語
      string::size_type ret = buf.find(" ");
      if( ret != buf.npos )
	buf = buf.substr(0,ret);
      _title_ac = buf;
    }

  }

  else if( buf.find(_de) == 0 ){
    _header = _de;
    buf = buf.substr(_header.length());

    if( ! _title_de.empty() )
      extend_title(_title_de,buf);
    else
      _title_de = trimLeftSpaces(buf);
  }

  else if( buf.find(_aaseq) == 0 ){
    _header = _aaseq;
    _aaseq_offset = ifs.tellg();
  }

  // 何かの継続行
  // 先頭が半角スペースの行は SEQUENCE の継続行
  else if( buf.find(" ") == 0 ){
    // 配列データの処理
    // ※ この統計処理は seqnew の実行結果に出るのみ
    if( _header == _aaseq ){
      trimSpaces(buf);
      int len = buf.length();

      // このエントリのアミノ酸配列の長さ
      _aaseqlen += len;

      // データベース全体のアミノ酸配列の長さ
      _total_aaseqlen += len;

      // 最長のアミノ酸配列
      if( _max_aaseqlen < _aaseqlen )
	_max_aaseqlen = _aaseqlen;

      // 最長の配列(GENESならアミノ酸と塩基の配列の長さの合計)
      _max_seqlen = _max_aaseqlen;

    }
  }

  // エントリ終了処理
  else if( buf.find(_delim) == 0 ){
    set_size(ifs);

    // ここまでが SEQUENCE行なら、ここで配列データの統計情報をセット
    if( _header == _aaseq )
      set_seq_size(buf,_aaseq_size,_aaseq_offset);

    writer();
  }

  // 何かのヘッダ行
  else{

    // ここまでが SEQUENCE行なら、ここで配列データの統計情報をセット
    if( _header == _aaseq ){
      _size = ifs.tellg();
      _size -= _offset;
      set_seq_size(buf,_aaseq_size,_aaseq_offset);
    }

    set_header(buf);
  }

}

void Swissprot::writer(){

  // プライマリ accession は大括弧で括る
  if( ! _title_ac.empty() ){
    _title = "[";
    _title += _title_ac;
    _title += "]";
  }

  // accession の後ろは半角スペースで連結
  if( ! _title.empty() && ! _title_de.empty() )
    _title += " ";

  _title += _title_de;

  _aaseq_id = "_aaseq_";
  _aaseq_id += _id;

  Seqnew::writer();

  if( _aaseq_offset > 0 )
    Seqnew::writer(_aaseq_id,_aaseq_offset,_aaseq_size);

  clear();
}

void Swissprot::clear(){

  Seqnew::clear();

  _title_ac = "";
  _title_de = "";
  _aaseq_id = "";
  _aaseq_offset = 0;
  _aaseq_size = 0;
  _aaseqlen = 0;

}
