#!/bin/bash

system=$(uname -s)
if [ $system = "Darwin" ]
then
  perl=$(which perl)
  if [ "$perl" != "/usr/local/bin/perl" ]
  then
    echo "Perl interface MCDB_File was skipped."
    rm -rf mcdb
    exit 0
  fi
fi

type realpath > /dev/null 2>&1
acc=$?
if [ $acc -ne 0 ]
then
  echo "Perl interface MCDB_File was skipped."
  rm -rf mcdb
  exit 0
fi

local=$(realpath $(dirname $0))/blib/local

elog=err.log
perl Makefile.PL 2> $elog
if [ -s $elog ]
then
  #cat $elog
  echo "Perl interface MCDB_File was skipped."
  rm $elog
  rm -rf mcdb
  exit 0
fi
rm $elog

sed="sed -i"
if [ $system = "Darwin" ]
then
  sed="sed -i .bak"
fi
$sed 's/^SITELIBEXP = .*/SITELIBEXP =/' Makefile
$sed 's/^SITEARCHEXP = .*/SITEARCHEXP =/' Makefile
$sed 's/^VENDORARCHEXP = .*/VENDORARCHEXP =/' Makefile
$sed 's/^VENDORLIBEXP = .*/VENDORLIBEXP =/' Makefile
$sed 's/^PERLPREFIX = .*/PERLPREFIX =/' Makefile
$sed 's/^SITEPREFIX = .*/SITEPREFIX =/' Makefile
$sed 's/^VENDORPREFIX = .*/VENDORPREFIX =/' Makefile
$sed 's/^INSTALLPRIVLIB = .*/INSTALLPRIVLIB = \/lib/' Makefile
$sed 's/^INSTALLSITELIB = .*/INSTALLSITELIB = \/lib/' Makefile
$sed 's/^INSTALLVENDORLIB = .*/INSTALLVENDORLIB = \/lib/' Makefile
$sed 's/^INSTALLARCHLIB = .*/INSTALLARCHLIB = \/lib/' Makefile
$sed 's/^INSTALLSITEARCH = .*/INSTALLSITEARCH = \/lib/' Makefile
$sed 's/^INSTALLVENDORARCH = .*/INSTALLVENDORARCH = \/lib/' Makefile
$sed 's/^INSTALLBIN = .*/INSTALLBIN = \/bin/' Makefile
$sed 's/^INSTALLSITEBIN = .*/INSTALLSITEBIN = \/bin/' Makefile
$sed 's/^INSTALLVENDORBIN = .*/INSTALLVENDORBIN = \/bin/' Makefile
$sed 's/^INSTALLSCRIPT = .*/INSTALLSCRIPT = \/bin/' Makefile
$sed 's/^INSTALLSITESCRIPT = .*/INSTALLSITESCRIPT = \/bin/' Makefile
$sed 's/^INSTALLVENDORSCRIPT = .*/INSTALLVENDORSCRIPT = \/bin/' Makefile
$sed 's/^INSTALLMAN1DIR = .*/INSTALLMAN1DIR = \/man\/man1/' Makefile
$sed 's/^INSTALLSITEMAN1DIR = .*/INSTALLSITEMAN1DIR = \/man\/man1/' Makefile
$sed 's/^INSTALLVENDORMAN1DIR = .*/INSTALLVENDORMAN1DIR = \/man\/man1/' Makefile
$sed 's/^INSTALLMAN3DIR = .*/INSTALLMAN3DIR = \/man\/man3/' Makefile
$sed 's/^INSTALLSITEMAN3DIR = .*/INSTALLSITEMAN3DIR = \/man\/man3/' Makefile
$sed 's/^INSTALLVENDORMAN3DIR = .*/INSTALLVENDORMAN3DIR = \/man\/man3/' Makefile

cc=
if [ $system = "Darwin" ]
then
  cc="--cc=clang"
  ver=$(sw_vers -productVersion|cut -d"." -f-2)
  $sed 's/^CC = .*/CC = clang/' Makefile
  $sed 's/^LD = .*/LD = clang/' Makefile
  $sed 's/^LDDLFLAGS = /LDDLFLAGS = -Wl,-w /' Makefile
  $sed 's/^LDFLAGS = /LDFLAGS = -Wl,-w /' Makefile
  $sed -E "s/-mmacosx-version-min=[^ ]+ /-mmacosx-version-min=$ver /" Makefile
  if [ -f Makefile.bak ]
  then
    rm Makefile.bak
  fi
fi

if [ ! -d mcdb ]
then
  mkdir mcdb
  cd ../../
  ./configure --optimize=-O3 $cc
  make
  cd -
  cp ../../*.h mcdb/
  cp ../../libmcdb.a mcdb/
  cp -r ../../plasma mcdb/
fi
make
make install DESTDIR=$local
